; The 700 Club
; Second edition
; /Mic, 2010, 2012

; Assemble with wla-dx


.include "snes.inc"
.include "filter_ids.inc"


 ; ZP variables
 .define channelToggles 	$0090
 .define playing 		$0096
 .define titleLen2		$0098
 .define authorLen2		$0099
 .define echoFlag		$009C
 .define filterNum		$009E
 
 .define joyData 		$00A0
 .define joyMask 		$00A2
 .define paused			$00A4
 .define scrollDelay2 		$00A5
 .define longTitleOffs 		$00A6
 .define longTitleLen		$00A8
 .define longTitleUpdated 	$00AA
 .define updateLongTitle 	$00AB
 
 .define upRepeatDelay		$00AC
 .define downRepeatDelay	$00AD
 
 .define framesPerSec 		$00B0
 .define scrollerTPosOld 	$00B2
 .define tempb2			$00B4
 .define mvolFade		$00B5
 .define secDelay		$00B6
 .define songListUpdated 	$00B7
 .define updateSongList		$00B8
 .define tempw2			$00B9
 .define ignoreLoops 		$00BB
 .define nextSong		$00BC
 
 .define volumeBars 		$00C0
 .define tempb			$00C5
 .define tempw			$00C6
 .define firstShown 		$00C8
 .define highlighted 		$00C9

 .define scrollerXPos 		$00CC
 .define scrollerTPos 		$00CE
 
 .define spcDataLen		$00D0
 .define indZp			$00D2
 .define lastByteSent 		$00D5
 .define titleOffs		$00D6
 .define authorOffs		$00D8
 .define gameOffs		$00DA
 .define scrollDelay1 		$00DC
 
 .define titleLen		$00E0
 .define authorLen		$00E2
 .define gameLen		$00E4
 
 .define putsColor	 	$00F8
 .define putsOffs	 	$00FA
 

; Tables / buffers
.define longTitleBuffer 	$7E6000
.define songListBuffer		$7E8000
.define playbackTimeString 	$7EB000
.define colorAddHdmaTable 	$7EA800
.define scrollHdmaTable 	$7EA000

.define NUM_SONGS		50

; Playback driver commands
.define CMD_PLAY		$21
.define CMD_PAUSE		$22
.define	CMD_STOP		$23
.define CMD_SEND_LOOP_MSGS	$24
.define CMD_RESTART_SONG	$25
.define	CMD_SET_MVOL 		$30
.define CMD_TOGGLE_CHN		$40
.define CMD_TOGGLE_ECHO		$50

.define KEY_REPEAT_SHORT_DELAY 25
.define KEY_REPEAT_INITIAL_DELAY 70


 .bank 0
 .section "MainCode"
 

 
 ; Zero page variables
 .equ spcSongNr		$f0
 .equ spcMirrorVal	$f1
 .equ spcSourceAddr	$f2	; Three bytes!
 
 
; ############################################ Macros ############################################

 .macro sendMusicBlockM ; srcSeg srcAddr destAddr len
 	; Store the source address \1:\2 in musicSourceAddr.
 	sep     #$20
 	lda     #\1
 	sta     spcSourceAddr + 2
 	rep     #$20
 	lda     #\2
 	sta     spcSourceAddr
 
 	; Store the destination address in x.
 	; Store the length in y.
 	rep     #$10
 	ldx     #\3
 	ldy     #\4
 	jsr     CopyBlockToSPC
 .endm
 
 
 .macro sendMusicBlockM2 ; srcSeg srcAddr destAddr len
	; Store the source address \1:\2 in musicSourceAddr.
	sep     #$20
	lda     #\1
	sta     spcSourceAddr + 2
	rep     #$20
	lda     #\2
	sta     spcSourceAddr

	; Store the destination address in x.
	; Store the length in y.
	rep     #$10
	ldx     #\3
	ldy     \4
	jsr     CopyBlockToSPC
 .endm
 
 
 .macro startSPCExecM ; startAddr
	rep     #$10
	ldx     #\1
	jsr     StartSPCExec
 .endm
 
 
 .macro waitForAudio0M
 	sta	spcMirrorVal
 	-: 	cmp REG_APUI00
 	bne 	-
 .endm
 
 
 .macro PUTSBG0 ; string, x, y, pal
 	rep 	#$10
 	sep	#$20
 	lda	#143
 	sta	putsOffs
 	ldx	#+
 	ldy	#(\3*64 + \2*2 + $800)
 	lda 	#\4
 	bra 	++
 	+:
 	.db 	\1,0
 	++:
 	jsr	Puts0
.endm


; ################################################################################################


; Wait for the next vblank to start
_wait_nmi:
	php
	sep	#$20
-:
	lda.l	REG_RDNMI
	bmi	-
-:
	lda.l	REG_RDNMI
	bpl	-
	plp
	rts


; IN:
; a = bank
; x = offs
; OUT:
; y = string length
strlen:
	php	
	rep		#$30
	sep		#$20
	stx		indZp
	sta		indZp+2
	ldy		#0
-:
	lda		[indZp],y
	beq		+
	iny
	bra		-
+:
	plp
	rts



; Calculate the lengths of the title, author and game name strings for the currently loaded song
UpdateStringLengths:
	php
	rep		#$30
	and		#$FF
	asl		a
	asl		a
	asl		a
	sta		tempw
	
	ldx		tempw
	lda.l	string_table+2,x	; title
	tax
	lda		#0
	jsr		strlen
	sty		titleLen

	ldx		tempw
	lda.l	string_table+4,x	; author
	tax
	lda		#0
	jsr		strlen
	sty		authorLen

	ldx		tempw
	lda.l	string_table,x		; game
	tax
	lda		#0
	jsr		strlen
	sty		gameLen
	tya
	clc
	adc		titleLen
	sta		longTitleLen
	
	stz		longTitleOffs
	plp
	rts
	
	

; ##################################### Button toggling (UI) #####################################

; IN:
; indZp pointer to the button data
PressOrDepressButton:
	rep		#$20
	ldy		#0
	lda		[indZp],y
	tax
	sta.l		REG_VRAM_ADDR_L
-:
	iny
	iny
	lda		[indZp],y
	sta		REG_VRAM_DATAW1
	cpy		#6
	bne		-

	txa
	clc
	adc		#$20		; Move to next row

	sta.l		REG_VRAM_ADDR_L
-:
	iny
	iny
	lda		[indZp],y
	sta		REG_VRAM_DATAW1
	cpy		#12
	bne		-
	rts


; Change the Play button graphic to its pressed state	
PressPlayButton:
	php
	rep		#$20
	lda		#pressed_play_button
	sta		indZp
	sep		#$20
	lda		#:pressed_play_button
	sta		indZp+2
	jsr		PressOrDepressButton
	plp
	rts

; Change the Play button graphic to its depressed state
DepressPlayButton:
	php
	rep		#$20
	lda		#depressed_play_button
	sta		indZp
	sep		#$20
	lda		#:depressed_play_button
	sta		indZp+2
	jsr		PressOrDepressButton
	plp
	rts

PressPauseButton:
	php
	rep		#$20
	lda		#pressed_pause_button
	sta		indZp
	sep		#$20
	lda		#:pressed_pause_button
	sta		indZp+2
	jsr		PressOrDepressButton
	plp
	rts

DepressPauseButton:
	php
	rep		#$20
	lda		#depressed_pause_button
	sta		indZp
	sep		#$20
	lda		#:depressed_pause_button
	sta		indZp+2
	jsr		PressOrDepressButton
	plp
	rts

PressRepeatButton:
	php
	rep		#$20
	lda		#pressed_repeat_button
	sta		indZp
	sep		#$20
	lda		#:pressed_repeat_button
	sta		indZp+2
	jsr		PressOrDepressButton
	plp
	rts

DepressRepeatButton:
	php
	rep		#$20
	lda		#depressed_repeat_button
	sta		indZp
	sep		#$20
	lda		#:depressed_repeat_button
	sta		indZp+2
	jsr		PressOrDepressButton
	plp
	rts	
	
; ################################################################################################


; ####################################### Volume handling ########################################


; Set master volume
SetMvol:
	php
	sei
	sep		#$20
	stz		REG_NMI_TIMEN
	lda		volumeBars		; The first volume bar contains the master volume setting 
	asl		a
	cmp		#16
	bne		+
	lda		#15
+:
	ora		#CMD_SET_MVOL
	sta		REG_APUI01
	lda		spcMirrorVal
	ina
	sta		lastByteSent
	sta		REG_APUI00
	waitForAudio0M	
	lda		#$80
	sta		REG_NMI_TIMEN	
	plp
	rts


UpdateVolumeBars:
	php
	sei
	sep		#$20
	;stz		REG_NMI_TIMEN
	rep 		#$30
	lda		#($400 + 1*32 + 26)	; BG0 is at $400
	sta		tempw
	sep		#$20

	lda		channelToggles+1
	asl		a
	asl		a
	asl		a
	asl		a
	ora		channelToggles
	sta		tempb2

	lda		REG_APUI02
	eor		#$FF
	and		tempb2
	sta		tempb
	and		#$F
	cmp		#$F
	bne		+
	lda		#8
	bra		++
+:
	lsr		a
++:
	sta		volumeBars+1
	lda		tempb
	lsr		a
	lsr		a
	lsr		a
	lsr		a
	cmp		#$F
	bne		+
	lda		#8
	bra		++
+:
	lsr		a
++:
	sta		volumeBars+2

	lda		channelToggles+3
	asl		a
	asl		a
	asl		a
	asl		a
	ora		channelToggles+2
	sta		tempb2

	lda		REG_APUI03
	eor		#$FF
	and		tempb2
	sta		tempb
	and		#$F
	cmp		#$F
	bne		+
	lda		#8
	bra		++
+:
	lsr		a
++:
	sta		volumeBars+3
	lda		tempb
	lsr		a
	lsr		a
	lsr		a
	lsr		a
	cmp		#$F
	bne		+
	lda		#8
	bra		++
+:
	lsr		a
++:
	sta		volumeBars+4
+++:

	stz		tempb
	ldy		#3
-:
	lda		tempw
	sta		REG_VRAM_ADDR_L	
	lda		tempw+1
	sta		REG_VRAM_ADDR_H	
	ldx		#0
--:
	lda		volumeBars,x
	clc
	adc		tempb
	sta		REG_VRAM_DATAW1
	lda		#(12 + 2)
	sta		REG_VRAM_DATAW2
	inx
	cpx		#5
	bne		--
	rep		#$20
	lda		tempw
	clc
	adc		#32
	sta		tempw
	sep		#$20
	lda		tempb
	clc
	adc		#9
	sta		tempb
	dey
	bne		-
	sep		#$20
	;lda		#$80
	;sta		REG_NMI_TIMEN	
	plp	
	rts
	
	
	
; ######################################## SPC transfer #########################################


; spcSourceAddr - source address
; x - dest address
; y - count
 CopyBlockToSPC:
 	rep		#$10
 	
	; Wait until audio0 is $aa
	sep     #$20
	lda     #$AA
	waitForAudio0M

	stx     REG_APUI02		; Write it to APU Port2 as well
	
	; Transfer count to x.
	phy
	plx

 	; Send $01cc to AUDIO0 and wait for echo.
	lda		#$01
	sta     REG_APUI01
	lda		#$CC
	sta     REG_APUI00
	waitForAudio0M

	sep     #$20

	; Zero counter.
	ldy     #$0000
 CopyBlockToSPC_loop:
	lda     [spcSourceAddr],y
	sta		REG_APUI01
	
	; Counter -> A
	tya
	sta		REG_APUI00

	; Wait for counter to echo back.
	waitForAudio0M

	; Update counter and number of bytes left to send.
	iny
	dex
	bne     CopyBlockToSPC_loop

 	sep		#$20
 	
	; Send the start of IPL ROM send routine as starting address.
	ldx     #$FFC9
	stx     REG_APUI02
	
	; Tell the SPC we're done transfering for now
	lda     #0
	sta		REG_APUI01
	lda		spcMirrorVal	; This value is filled out by waitForAudio0M
	clc
	adc		#$02		
	sta		REG_APUI00

	; Wait for counter to echo back.
	waitForAudio0M

	rep     #$20
	
	rts

 
 
; Starting address is in x.
StartSPCExec:
	; Wait until audio0 is $aa
	sep     #$20
	lda     #$AA
	waitForAudio0M

	; Send the destination address to AUDIO2.
	stx     REG_APUI02

	; Send $00cc to AUDIO0 and wait for echo.
	lda     #$00
	sta     REG_APUI01
	lda     #$CC
	sta     REG_APUI00
	waitForAudio0M

	rts



LoadVgm:
	php
	sep		#$20
 	stz     	REG_NMI_TIMEN   
 	sei        	
	jsr		_wait_nmi
	jsr		ClearLcd
	PUTSBG0 	"Loading..",1,1,2
	
	rep		#$30
	lda		nextSong
	and		#$FF
	asl		a
	asl		a
	asl		a
	sta		tempw
	asl		a
	tax
	lda.l		vgm_table,x
	sta		spcDataLen
	lda.l		vgm_table+8,x
	clc
	adc		spcDataLen
	sta		spcDataLen
	
	; Load the first 32 kB (or less) of the VGM
	lda.l		vgm_table+2,x
	sta		indZp
	sep		#$20
	lda.l		vgm_table+4,x
	sta		indZp+2
	rep		#$20
	lda.l		vgm_table,x
	phx
	tax
	sep		#$20
-:
	dex
	bmi		+
	txy
	lda		[indZp],y
	sta.l		$7F0000,x
	bra		-
+
	plx

	; Load the remainder of the VGM (if any)	
	lda.l		vgm_table+10,x
	sta		indZp
	sep		#$20
	lda.l		vgm_table+12,x
	sta		indZp+2
	rep		#$20
	lda.l		vgm_table+8,x
	tax
	sep		#$20
-:
	dex
	bmi		+
	txy
	lda		[indZp],y
	sta.l		$7F8000,x
	bra		-
+

	; Send the entire VGM to SPC RAM at $1800
	sendMusicBlockM2 $7f, $0000, $1800, spcDataLen

  	rep		#$10	       ; xy in 16-bit mode.
  	sep		#$20
  	ldx.w 		#$E8
  -:
  	lda.l 		player,x     	; Copy player code
  	sta.l 		$7F0000,x
  	dex
 	bpl 		-
 	lda		#echoFlag
 	sta.l		$7F0047
 	lda		filterNum
 	sta.l		$7F0048
 	sendMusicBlockM $7F, $0047, $0047, $00A8
 	
	; The playback driver entrypoint is at $0300 in SPC RAM
	startSPCExecM $0300
	
	; The playback driver will write $CD to Port0 when it's entering its command-reading loop - so
	; wait for that.
	sep		#$20
	lda		#$CD
-:
	cmp 		REG_APUI00
 	bne 		-
	
	; Tell the driver to start playback
	lda		#CMD_PLAY
	sta		REG_APUI01
	lda		#$CE
	sta		lastByteSent
	sta		REG_APUI00
	waitForAudio0M

	sep		#$20
	lda		volumeBars
	asl		a
	cmp		#16
	bne		+
	lda		#15
+:
	ora		#CMD_SET_MVOL
	sta		REG_APUI01
	lda		#$CF
	sta		lastByteSent
	sta		REG_APUI00
	waitForAudio0M
	
	lda		framesPerSec
	sta		secDelay

	lda		ignoreLoops
	bne		+
	lda		#CMD_SEND_LOOP_MSGS
	sta		REG_APUI01
	lda		spcMirrorVal
	ina
	sta		REG_APUI00
	waitForAudio0M
+:

	lda		channelToggles+0
	bne		+
	lda		#0
	jsr		ToggleChannel
	stz		channelToggles+0
+:
	lda		channelToggles+1
	bne		+
	lda		#1
	jsr		ToggleChannel
	stz		channelToggles+1
+:
	lda		channelToggles+2
	bne		+
	lda		#2
	jsr		ToggleChannel
	stz		channelToggles+2
+:
	lda		channelToggles+3
	bne		+
	lda		#3
	jsr		ToggleChannel
	stz		channelToggles+3
+:

	jsr		_wait_nmi
	lda		nextSong
	sta		playing
	stz		paused
	stz		titleOffs
	stz		authorOffs
	stz		longTitleOffs
	stz		titleOffs+1
	stz		authorOffs+1
	stz		longTitleOffs+1
	jsr		PutsTitle
	jsr		PutsAuthor
	jsr		PressPlayButton
	jsr		DepressPauseButton
	
	sep		#$20
	lda		#'0'
	sta.l		playbackTimeString
	sta.l		playbackTimeString+1
	sta.l		playbackTimeString+3
	sta.l		playbackTimeString+4
	lda		#':'
	sta.l		playbackTimeString+2
	lda		#0
	sta.l		playbackTimeString+5
	jsr		PutsTime
	
	lda		framesPerSec
	clc
	adc		framesPerSec
	adc		framesPerSec
	sta		scrollDelay1

	lda		authorLen
	sta		authorLen2
	lda		titleLen
	sta		titleLen2
	
	sep		#$20	
 	cli             ; Enable IRQ
 	lda     	#$80
 	sta     	REG_NMI_TIMEN
 	plp
 	rts
 	
	
 
InitVgmPlayer:
 	sep		#$20
  	; Make sure NMI and IRQ are off
 	stz     	REG_NMI_TIMEN   
 	sei             	
 
 	; Make sure echo is off
 	sep		#$20
 	lda		#$7d		; Register $7d (EDL)
 	sta.l		$7f0100		; Just store the value somewhere in RAM temporarily..
 	lda		#$00		
 	sta.l		$7f0101
 	sendMusicBlockM $7f, $0100, $00f2, $0002 	; Send 2 bytes from $7f0100 to $f2,$f3 on the SPC (i.e. the DSP ADDR/DATA registers)
 	sep		#$20
 	lda		#$6C		; Register $6C (FLG)
 	sta.l		$7F0100
 	lda		#$20		; Bit 5 on (~ECEN)	
 	sta.l		$7F0101
 	sendMusicBlockM $7F, $0100, $00F2, $0002
 
  	rep		#$10	       ; xy in 16-bit mode.
  	sep		#$20
  	ldx.w 		#$17FF ;6FF
  -:
  	lda.l 		player,x     	; Copy player code
  	sta.l 		$7F0000,x
  	dex
 	bpl 		-

 	sendMusicBlockM $7F, $0047, $0047, $00A8	
 	sendMusicBlockM $7F, $0100, $0100, $1700 
 	
 	rts
 	
 	
 	
; ######################################## Driver config ########################################


; Toggle song looping on/off	
ToggleLooping:
	php
	sei
	sep		#$20
	stz     	REG_NMI_TIMEN   
	lda		#CMD_SEND_LOOP_MSGS
	sta		REG_APUI01
	lda		spcMirrorVal
	ina
	sta		REG_APUI00
	waitForAudio0M
	lda		ignoreLoops
	eor		#1
	sta		ignoreLoops
	jsr		_wait_nmi
	lda		ignoreLoops
	bne		+
	jsr		DepressRepeatButton
	bra		++
+:
	jsr		PressRepeatButton
++:
	lda		#$80
	sta		REG_NMI_TIMEN
	plp
	rts


; Toggle channels on/off
ToggleChannel:
	php
	sei
	sep		#$20
	stz     	REG_NMI_TIMEN
	sta		tempb
	ora		#CMD_TOGGLE_CHN
	sta		REG_APUI01
	lda		spcMirrorVal
	ina
	sta		REG_APUI00
	waitForAudio0M
	rep		#$20
	lda		tempb
	and		#$ff
	tax
	sep		#$20
	lda		channelToggles,x
	eor		#15
	sta		channelToggles,x
	lda		#$80
	sta		REG_NMI_TIMEN	
	plp
	rts
	

; Restart the currently playing song	
RestartVgm:
	php
	sei
	sep		#$20
	stz    	 	REG_NMI_TIMEN
	lda		#CMD_PLAY
	sta		REG_APUI01
	lda		spcMirrorVal
	ina
	sta		REG_APUI00
	waitForAudio0M
	; Song isn't paused anymore. Make sure the UI is updated
	jsr		DepressPauseButton
	jsr		PressPlayButton
	lda		#$80
	sta		REG_NMI_TIMEN	
	plp
	rts
	
	
; Pause/unpause the currently playing song	
PauseVgm:
	php
	sei
	sep		#$20
	stz     	REG_NMI_TIMEN
	lda		#$22
	sta		REG_APUI01
	lda		spcMirrorVal
	ina
	sta		REG_APUI00
	waitForAudio0M
	lda		paused
	eor		#1
	sta		paused
	jsr		_wait_nmi
	lda		paused
	bne		+
	jsr		DepressPauseButton
	jsr		PressPlayButton
	bra		++
+:
	jsr		PressPauseButton
	jsr		DepressPlayButton
++:
	lda		#$80
	sta		REG_NMI_TIMEN	
	plp
	rts


; Stop the currently playing song	
StopVgm:
	php
	sei
	sep		#$20
	stz    		REG_NMI_TIMEN
	jsr		_wait_nmi
	jsr		DepressPlayButton
	lda		#CMD_STOP
	sta		REG_APUI01
	lda		spcMirrorVal
	ina
	sta		REG_APUI00
	plp
	rts
	
NextFilter:
	php
	sei
	sep		#$20
	stz    		REG_NMI_TIMEN
	inc		filterNum
	lda		filterNum
	cmp		#FILTER_NONE
	bcc		+
	cmp		#FILTER_INVALID
	bne		++
	stz		filterNum
	stz		echoFlag
	bra		+
++:
	lda		#$20
	sta		echoFlag
+:
	lda		#CMD_TOGGLE_ECHO
	sta		REG_APUI01
	lda		spcMirrorVal
	ina
	sta		REG_APUI00
	waitForAudio0M
	lda		#$80
	sta		REG_NMI_TIMEN	
	plp
	rts
	

; ################################################################################################


LoadVram:
	php
	rep		#$20
	lda		9,s		; vramOffs
	sta.l		REG_VRAM_ADDR_L
	lda		11,s		; numBytes
	sta.l		REG_DAS0L
	lda		5,s		; src (lower 16 bits)
	sta.l		REG_A1T0L
	sep		#$20
	lda		7,s		; src bank
	sta.l		REG_A1B0
	lda		#1
	sta.l		REG_DMAP0
	lda		#$80
	sta.l		REG_VRAM_INC
	lda		#$18		; PPU register to DMA to
	sta.l		REG_BBAD0
	lda		#1		; Enable main DMA
	sta.l		REG_MDMAEN
	plp
	rtl


HdmaChn1:
	php
	rep		#$20
	lda		#tilesize_hdma_table
	sta.l		REG_A1T1L
	sep		#$20
	lda		#:tilesize_hdma_table
	sta.l		REG_A1B1
	lda		#0		; transfer mode (1-address)
	sta.l		REG_DMAP1
	lda		#$80
	sta.l		REG_VRAM_INC
	lda		#$05		; register to write to (DISPMODE)
	sta.l		REG_BBAD1
	plp
	rts


HdmaChn2:
	php
	rep		#$20
	lda		#$A000		; offset(scrollHdmaTable)
	sta.l		$4322
	sep		#$20
	lda		#$7E		; bank(scrollHdmaTable)
	sta.l		$4324
	lda		#3		; transfer mode (2-address, write twice)
	sta.l		$4320
	lda		#$80
	sta.l		REG_VRAM_INC
	lda		#$0D		; register to write to (BG0HOFS)
	sta.l		$4321
	plp
	rts


HdmaChn3:
	php
	rep		#$20
	lda		#$A800		; offset(colorAddHdmaTable)
	sta.l		REG_A1T3L	; table address (offset)
	sep		#$20
	lda		#$7E		; bank(colorAddHdmaTable)
	sta.l		REG_A1B3	; table address (bank)
	lda		#0		; transfer mode (1-address)
	sta.l		REG_DMAP3
	lda		#$80
	sta.l		REG_VRAM_INC
	lda		#$31		; register to write to (CGADSUB)
	sta.l		REG_BBAD3
	plp
	rts
	

; ################################################################################################


; Clear the "LCD" (the upper-left part of the screen that shows the artist, 
; playback time, etc).
ClearLcd:
	php
	rep		#$30
	lda		#(1*32 + 1 + $400)
	sta		REG_VRAM_ADDR_L
	jsr		_clear_lcd_helper
	rep		#$30
	lda		#(2*32 + 1 + $400)
	sta		REG_VRAM_ADDR_L
	jsr		_clear_lcd_helper
	rep		#$30
	lda		#(3*32 + 1 + $400)
	sta		REG_VRAM_ADDR_L
	jsr		_clear_lcd_helper
	plp
	rts

_clear_lcd_helper:
	ldx 		#22
	sep		#$20
-:
	lda		#144
	sta		REG_VRAM_DATAW1
	lda		#8
	sta		REG_VRAM_DATAW2
	dex
	bne		-
	rts	
	

ClearBG0:
	php
	rep 		#$30
	lda		#$400
	sep		#$20
	sta		REG_VRAM_ADDR_L	
	xba
	sta		REG_VRAM_ADDR_H	
	ldx		#$400
-:		
	lda		#0 
	sta		REG_VRAM_DATAW1
	lda 		#$11 
	sta 		REG_VRAM_DATAW2
	dex
	bne		-
	plp
	rts


ClearBG1:
	php
	rep 		#$30
	lda		#$000
	sep		#$20
	sta		REG_VRAM_ADDR_L	
	xba
	sta		REG_VRAM_ADDR_H	
	ldx		#$400
-:		
	lda		#144
	sta		REG_VRAM_DATAW1
	stz 		REG_VRAM_DATAW2
	dex
	bne		-
	plp
	rts

	
ClearBG2:
	php
	rep 		#$30
	lda		#$C00
	sep		#$20
	sta		REG_VRAM_ADDR_L	
	xba	
	sta		REG_VRAM_ADDR_H	
	ldx		#$300
-:		
	lda		#0
	sta		REG_VRAM_DATAW1
	stz 		REG_VRAM_DATAW2
	dex
	bne		-
	ldx		#$100
-:		
	lda		#192
	sta		REG_VRAM_DATAW1
	stz 		REG_VRAM_DATAW2
	dex
	bne		-
	plp
	rts


ClearOAM:
	php
	rep		#$10
	sep		#$20
	stz		REG_OAMADDL
	stz		REG_OAMADDH
	ldx		#544
-:
	stz		REG_OAMDATA
	dex
	bne		-
	plp
	rts


; The current audio filter graphic is shown using 12 8x8 pixel sprites
UpdateOAM:
	php
	sei
	sep		#$20
	stz    		REG_NMI_TIMEN
	jsr		_wait_nmi
	
	rep		#$30
	lda		filterNum
	and		#$F
	asl		a
	asl		a
	sta		tempw
	asl		a
	adc		tempw		; A = filterNum*12
	sta		tempw
	sep		#$20
	stz		REG_OAMADDL
	lda		#$80
	sta		REG_OAMADDH

	ldy		#6		; Update the 6 sprites making up the top half
	lda		#206
-:
	sta		REG_OAMDATA
	pha
	lda		#40
	sta		REG_OAMDATA
	lda		tempw
	sta		REG_OAMDATA
	lda		tempw+1
	and		#1
	ora		#$30
	sta		REG_OAMDATA
	rep		#$20
	inc		tempw
	sep		#$20
	pla
	clc
	adc		#8
	dey
	bne		-

	ldy		#6		; Update the 6 sprites making up the bottom half
	lda		#206
-:
	sta		REG_OAMDATA
	pha
	lda		#48
	sta		REG_OAMDATA
	lda		tempw
	sta		REG_OAMDATA
	lda		tempw+1
	and		#1
	ora		#$30
	sta		REG_OAMDATA
	rep		#$20
	inc		tempw
	sep		#$20
	pla
	clc
	adc		#8
	dey
	bne		-
	
	lda		#$80
	sta    		REG_NMI_TIMEN
	plp	
	rts	
	

; ################################################################################################


; Updates the song list shown on screen. To avoid having to execute this routine during vblank it writes the tile map
; data to a buffer in RAM, which then is sent to VRAM via DMA during the next vblank.
PutsSongs:
	php
	rep		#$30
	lda		#2
	sta		tempw
	sep		#$20
	stz		tempb2
	lda		#16
	sta		putsColor
	lda		firstShown
	and		#1
	beq		+
	lda		#28
	sta		putsColor
+:
	phb
	lda		#$7E		; Set DBR=$7E (all 16-bit addresses will implicitly refer to bank $7E (i.e. RAM))
	pha
	plb
psloop:
	rep		#$20
	lda		tempw
	tax
	clc
	adc		#64
	sta		tempw
	sep		#$20
	ldy		#29
-:
	lda		#$0E
	sta.w		$8000,x		; Write to $7E8000,x (i.e. the songListBuffer)
	lda 		putsColor
	eor		#12
	clc
	adc		#1
	inx
	sta.w		$8000,x
	inx
	dey
	bne		-
	
	lda		#0
	xba
	lda		tempb2
	cmp		#8
	bcc		+
	rep		#$20
	lda		#spaces
	bra		++
+:
	clc
	adc		firstShown
	rep		#$20
	asl		a
	asl		a
	asl		a
	tax
	lda.l 		string_table,x
	clc
++:
	sta		indZp
	sep		#$20
	stz		indZp+2
	ldy		#0

	phx
	rep		#$20
	lda		tempw
	tax
	clc
	adc		#64
	sta		tempw
	sep		#$20	
-:
	lda		[indZp],y
	beq		+
	clc
	adc		#(176-32)
	sta.w		$8000,x
	inx
	lda		putsColor
	adc		#0
	sta.w		$8000,x
	inx
	iny
	cpy		#29
	bne		-
+:
	lda		tempb2
	cmp		#8
	bcc		+
	rep		#$20
	lda		#spaces
	bra		++
+:
	rep		#$20
	pla
	cpy		#29
	beq		+
	phx
	tax
	lda.l 		string_table+2,x
	clc
++:
	sta		indZp
	sep		#$20
	stz		indZp+2
	sty		tempw2
	ldy		#0
	plx
-:
	lda		[indZp],y
	beq		++
	clc
	adc		#(176-32)
	sta.w		$8000,x
	inx
	lda		putsColor
	adc		#0
	sta.w		$8000,x
	inx
	iny
	inc		tempw2
	lda		tempw2
	cmp		#29
	bne		-
++:
	ldy		tempw2
+:
	sep		#$20
	cpy		#29
	beq		+
-:
	lda		#176
	sta.w		$8000,x
	inx
	lda		putsColor
	adc		#0
	sta.w		$8000,x
	inx
	iny
	cpy		#29
	bne		-
+:
	lda		#1
	sta		songListUpdated
	lda		putsColor
	eor		#12
	sta		putsColor
	inc		tempb2
	lda		tempb2
	cmp		#9
	beq		+
	jmp		psloop
+:
	plb
	plp
	rts
	

	
PutsTitle:
	php
	rep		#$30
	lda		#(1*32 + 1 + $400)		; BG0 is at $400
	sta		REG_VRAM_ADDR_L
	lda		playing
	and		#$FF
	asl		a
	asl		a
	asl		a
	tax
	lda.l 		string_table+2,x
	clc
	adc 		titleOffs
	sta		indZp
	sep		#$20
	stz		indZp+2
	ldy		#0
-:
	lda		[indZp],y
	beq		+
	clc
	adc		#112
	sta		REG_VRAM_DATAW1
	lda		#8
	adc		#0
	sta		REG_VRAM_DATAW2
	iny
	cpy		#22
	bne		-
+:
	cpy		#22
	beq		+
-:
	lda		#144
	sta		REG_VRAM_DATAW1
	lda		#8
	sta		REG_VRAM_DATAW2
	iny
	cpy		#22
	bne		-
+:
	plp
	rts


PutsAuthor:
	php	
	rep		#$30
	lda		#(2*32 + 1 + $400)		; BG0 is at $400
	sta		REG_VRAM_ADDR_L
	lda		playing
	and		#$FF
	asl		a
	asl		a
	asl		a
	tax
	lda.l 	string_table+4,x
	clc
	adc 	authorOffs
	sta		indZp
	sep		#$20
	inc		$97
	stz		indZp+2
	ldy		#0
-:
	lda		[indZp],y
	beq		+
	clc
	adc		#112
	sta		REG_VRAM_DATAW1
	lda		#8
	adc		#0
	sta		REG_VRAM_DATAW2
	iny
	cpy		#22
	bne		-
+:
	cpy		#22
	beq		+
-:
	lda		#144
	sta		REG_VRAM_DATAW1
	lda		#8
	sta		REG_VRAM_DATAW2
	iny	
	cpy		#22
	bne		-
+:
	plp
	rts


PutsLongTitle:
	php	
	rep		#$30

	lda		highlighted
	and		#$FF
	asl		a
	asl		a
	asl		a
	tax

	stz		tempw2
	
	sep		#$20
	lda		#16
	sta		putsColor
	lda		highlighted
	and		#1
	beq		+
	lda		#28
	sta		putsColor
+:

	stz		tempw
	stz		tempw+1
	lda		longTitleOffs
	sec
	sbc		gameLen
	bcc		+
	sta		tempw
	ldy		#0
	bra		++
+:
	rep		#$20
	lda.l 		string_table,x
	phx
	clc
	adc 		longTitleOffs
	sta		indZp
	sep		#$20
	stz		indZp+2
	ldy		#0
	ldx		#0
-:
	lda		[indZp],y
	beq		+++
	clc
	adc		#144
	sta.l		longTitleBuffer,x
	inx
	lda		putsColor
	adc		#0
	sta.l		longTitleBuffer,x
	inx
	iny
	cpy		#29
	bne		-
+++:
	stx		tempw2
	plx
+:
	cpy		#29
	beq		+
++:
	rep		#$20
	lda.l 		string_table+2,x
	clc
	adc 		tempw
	sta		indZp
	sep		#$20
	stz		indZp+2
	sty		tempw
	ldy		#0
	ldx		tempw2
-:
	lda		[indZp],y
	beq		++
	clc
	adc		#144
	sta.l		longTitleBuffer,x
	inx
	lda		putsColor
	adc		#0
	sta.l		longTitleBuffer,x
	inx
	iny
	inc		tempw
	lda		tempw
	cmp		#29
	bne		-
++:
	ldy		tempw
+:
	cpy		#29
	beq		+
-:
	lda		#176
	sta.l		longTitleBuffer,x
	inx
	lda		putsColor
	sta.l		longTitleBuffer,x
	inx
	iny	
	cpy		#29
	bne		-
+:
	lda		#1
	sta		longTitleUpdated
	plp
	rts


; a = palette
; x = source address
; y = dest address
Puts0:
	rep		#$10
	sep		#$20
	asl		a
	asl		a
	sta		putsColor
	rep		#$20
	tya
	lsr		a
	sep		#$20
	sta		REG_VRAM_ADDR_L	
	xba
	sta		REG_VRAM_ADDR_H	
	phy
	ldy		#20	
	-:
	lda.w		$0000,x
	beq		Puts0Done
	sec
	sbc		#32
	adc		putsOffs
	sta		REG_VRAM_DATAW1
	lda		putsColor
	sta		REG_VRAM_DATAW2
	inx
	dey
	bne		-
	Puts0Done:
	ply
	rts


PutsTime:
	php
	rep		#$10
	ldy		#(3*32 + 1 + $400)		; BG0 is at $400
	rep		#$20
	tya
	sep		#$20
	sta		REG_VRAM_ADDR_L	
	xba
	sta		REG_VRAM_ADDR_H	
	phy
	ldy		#5
	ldx		#0
	-:
	lda.l		playbackTimeString,x
	beq		+
	clc
	adc		#112
	sta		REG_VRAM_DATAW1
	lda		#8
	sta		REG_VRAM_DATAW2
	inx
	dey
	bne		-
+:
	ply
	plp
	rts
	

; ################################################################################################


; The scrolltext is implemented as follows:
;
; The first 16 characters of the string are placed "off-screen" on the BG0 map. The scrolltext
; uses 16x16 pixel tiles, so those 16 characters fill up the entire 256-pixel horizontal span
; of the screen.
; During vblank the HOFS value of BG0 is updated until it has been moved 256 pixels and all of
; those 16 characters are on-screen. At that point CopyScroller is called to draw the same 
; characters again, but on the other half of the map (the map is 32 tiles, or 512 pixels, wide).
; Then PutsScroller is called again to draw the next 16 characters on what is now the visible part
; of the map. Finally, the scrolling is reset so that the old characters are visible while the new
; ones are off-screen and will be scrolled in gradually.

CopyScroller:
	php
	rep		#$30
	lda		#(11*32 + 0 + $400)		; BG0 is at $400
	sta		REG_VRAM_ADDR_L
	lda		scrollerTPosOld
	asl		a
	tax
	ldy		#16
	sep		#$20
-:
	rep		#$20
	lda.l		scrolltext,x
	bne		+
	ldx		#0						; wrap
	bra		-
+:
	sep		#$20
	sta		REG_VRAM_DATAW1
	xba
	sta		REG_VRAM_DATAW2
	inx
	inx
	dey
	bne		-
	plp
	rts


PutsScroller:
	php
	rep		#$30
	lda		#(11*32 + 16 + $400)	; BG0 is at $400
	sta		REG_VRAM_ADDR_L
	lda		scrollerTPos
	sta		scrollerTPosOld
	asl		a
	tax
	ldy		#16
	sep		#$20
-:
	rep		#$20
	inc		scrollerTPos
	lda.l		scrolltext,x
	bne		+
	stz		scrollerTPos			; wrap
	ldx		#0
	bra		-
+:
	sep		#$20
	sta		REG_VRAM_DATAW1
	xba
	sta		REG_VRAM_DATAW2
	inx
	inx
	dey
	bne		-
	rep		#$20
	lda		#$0000
	sta.l		scrollHdmaTable+6
	plp
	rts
	
 
; ################################################################################################


MoveToNextSong:
	php
	sep		#$20
	lda		highlighted
	ina
	cmp		#NUM_SONGS
	bcs		MTNG_Done
	sta		highlighted
	lda		framesPerSec
	asl		a
	clc
	adc		framesPerSec
	sta		scrollDelay2
	rep		#$20
	lda		highlighted
	jsr		UpdateStringLengths
	sep		#$20
	
	lda		firstShown
	clc
	adc		#3
	cmp		highlighted
	bcs		+ 
	lda		firstShown
	clc
	adc		#8
	cmp		#NUM_SONGS
	bcs		+ 
	inc		firstShown
+:
	lda		#1
	sta		updateSongList
	stz		longTitleOffs
	stz		longTitleOffs+1
	lda		#1
	sta		updateLongTitle
MTNG_Done:
	plp
	rts


MoveToPrevSong:
	php
	sep		#$20
	lda		highlighted
	dea
	bmi		MTPG_Done
	sta		highlighted
	lda		framesPerSec
	asl		a
	clc
	adc		framesPerSec
	sta		scrollDelay2
	rep		#$20
	lda		highlighted
	jsr		UpdateStringLengths
	sep		#$20
	
	lda		firstShown
	beq		+ 
	clc
	adc		#3
	cmp		highlighted
	bcc		+ 
   	dec		firstShown
+:
	lda		#1
	sta		updateSongList
	stz		longTitleOffs
	stz		longTitleOffs+1
	lda		#1
	sta		updateLongTitle
MTPG_Done:
	plp
	rts
	

; ################################################################################################



 Start:
	; Initialize the SNES.
	Snes_Init

	sep		#$30

	lda     	#BLANK_SCREEN  	; Force VBlank by turning off the screen.
	sta     	REG_DISPCNT

	lda		#15
	sta		channelToggles
	sta		channelToggles+1
	sta		channelToggles+2
	sta		channelToggles+3
	
	lda		#60
	sta		framesPerSec
	lda		REG_STAT78
	and		#$10
	beq		+
	lda		#50
	sta		framesPerSec
+:

	sep		#$20
	lda		#FILTER_NONE
	sta		filterNum
	lda		#$20
	sta		echoFlag	
	
	jsr		InitVgmPlayer

	; Decrunch pattern data to RAM at $7f6000
	sep 		#$20
	lda 		#$7f
	phb 					; Save old DBR 
	pha
	plb
	lda		#$01
	rep 		#$10
	ldx 		#$6000			; Decode to $7f6000
	ldy 		#patterns
	jsr 		aplib_decrunch
	plb					; Restore old DBR

	; DMA the decrunched patterns to VRAM
	rep 		#$30
	pea		(72*32)			; Bytes to copy
	pea		$4000			; VRAM word address
	pea		$007F			; Source bank
	pea		$6000			; Source offset
	jsl		LoadVram

	pea		512			; Bytes to copy
	pea		$4000 + 72*16		; VRAM word address
	pea		:patterns_end
	pea		patterns_end
	jsl		LoadVram

	pea		112*16
	pea		$4000 + 72*16 + 256
	pea		:font2bpp
	pea		font2bpp
	jsl		LoadVram

	pea		112*32
	pea		$4000 + 72*16 + 256 + 112*8
	pea		:font4bpp
	pea		font4bpp
	jsl		LoadVram

	pea		27*32
	pea		$4000 + 72*16 + 256 + 112*8 + 112*16 + 4096
	pea		:volume_bars
	pea		volume_bars
	jsl		LoadVram

	sep 		#$20
	lda 		#$7f
	phb 					; Save old DBR 
	pha
	plb
	lda		#$40
	rep 		#$10
	ldx 		#$6000			; Decode to $7f6000
	ldy 		#bigfont
	jsr 		aplib_decrunch
	plb					; Restore old DBR

	; DMA to VRAM
	rep 		#$30
	pea		8192
	pea		$4000 + 72*16 + 256 + 112*8 + 112*16
	pea		$007F
	pea		$6000
	jsl		LoadVram

	pea		1920
	pea		$2000
	pea		:filter_buttons
	pea		filter_buttons
	jsl		LoadVram
	
	SetPalette 	palette,0,32
	SetPalette 	font2bpp_colors,17,6
	SetPalette 	palette_end,20,30
	SetPalette 	font2bpp_colors2,29,6
	SetPalette 	font4bpp_colors,32,4
	SetPalette 	volume_pal,48,32
	SetPalette 	bigfont_colors,64,24
	SetPalette	filter_buttons_colors,128,16
	
	jsr		ClearBG1
	
	pea		256			; Bytes to copy
	pea		$0000			; VRAM word address (BG1)
	pea		:map_end
	pea		map_end
	jsl		LoadVram
 	
	pea		(map_end-map)		; Bytes to copy
	pea		$0080			; VRAM word address (BG1)
	pea		:map
	pea		map
	jsl		LoadVram

	; Hide some of the buttons (both the pressed and depressed buttons are present in the 
	; map to simplyfy the graphics conversion process).
	rep		#$20
	lda		#$00AA
	sta.l		REG_VRAM_ADDR_L
	sep		#$20
	ldx		#9
-:
	lda		#24
	sta		REG_VRAM_DATAW1
	lda		#0
	sta		REG_VRAM_DATAW2
	dex
	bne		-
	rep		#$20
	lda		#$00CA
	sta.l		REG_VRAM_ADDR_L
	sep		#$20
	ldx		#9
-:
	lda		#42
	sta		REG_VRAM_DATAW1
	lda		#0
	sta		REG_VRAM_DATAW2
	dex
	bne		-

	jsr		ClearBG0
	jsr		ClearBG2
	jsr		ClearOAM
	
	sep     	#$20

	lda		#7
	sta		volumeBars
	stz		volumeBars+2
	stz		volumeBars+1
	stz		volumeBars+4
	stz		volumeBars+3

	lda		#150
	sta		scrollDelay1
	sta		scrollDelay2
	
	lda		#22
	sta		firstShown
	lda		#25
	sta		highlighted
	sta		nextSong
	stz		playing
	
	lda		#1
	sta		ignoreLoops

	rep		#$20
	lda		nextSong
	jsr		UpdateStringLengths
	sep		#$20
	
	jsr		UpdateVolumeBars
	
	sep		#$20
	; Set display mode 1 
	lda		#1
	sta     	REG_DISPMODE

	; Enable BG0,1,2
	;
	; BG 0 and 1 are 4bpp, BG2 is 2bpp
	;
	; BG0 is used for the text on the "LCD" (song title, artist, playback time), the volume
	; bars and the scrolltext. The scrolltext uses 16x16 pixel tiles, while the rest of BG0
	; uses 8x8 pixel tiles, so a HDMA is used for writing to DISPMODE mid-screen in order to
	; change the tile size. Another HDMA is used to change the HOFS/VOFS parameters mid-screen
	; in order to move the scrolltext.
	;
	; BG1 is used for the "LCD" background, the panels, and the buttons.
	;
	; BG2 is used for the song list. A HDMA channel is used to toggle the hardware color addition
	; effect mid-screen to highlight the user-selected song.
	;
	lda     	#(BG2_ENABLE + BG1_ENABLE + BG0_ENABLE + OBJ_ENABLE)
	sta     	REG_BGCNT

	; Set the pattern base address for BG0 and BG1 to $8000
	lda		#$44
	sta		REG_CHRBASE_L
	sta		REG_CHRBASE_H

	lda		#$FC
	sta		REG_BG2HOFS
	lda		#0
	sta		REG_BG2HOFS
	lda		#$4
	sta		REG_BG2VOFS
	lda		#0
	sta		REG_BG2VOFS
	
	; Set the map base address for BG0 to (word address) $0400 and the map size to 32x32 tiles
	lda		#4
	sta		REG_BG0MAP
	lda		#12
	sta		REG_BG2MAP
	stz    	 	REG_BG1MAP

	stz		joyMask
	stz		joyMask
	
	stz		songListUpdated
	
	stz		scrollerTPos
	stz		scrollerTPos+1
	stz		scrollerXPos
	stz		scrollerXPos+1

	jsr		PutsScroller

	; Clear the song list buffer
    	ldx		#1087 ;23
-:
	lda		#176
	sta.l		songListBuffer,x
	dex
	bpl		-
	
	jsr		PutsSongs

	sep		#$20
	lda		#'0'
	sta.l		playbackTimeString
	sta.l		playbackTimeString+1
	sta.l		playbackTimeString+3
	sta.l		playbackTimeString+4
	lda		#':'
	sta.l		playbackTimeString+2
	lda		#0
	sta.l		playbackTimeString+5

	lda     	#$2c 
	sta.l   	REG_COLDATA
	lda     	#$49 
	sta.l   	REG_COLDATA
	lda     	#$81
	sta.l  		REG_COLDATA
	lda		#0
	sta.l		REG_CGWSEL
	lda		#$44
	sta.l		REG_CGADSUB
    
    	jsr		PressRepeatButton
    
	; Color Add hdma table in RAM
	rep		#$20
	lda		#$4431			; LSB: scanline delay, MSB: REG_CGADSUB setting
	sta.l		colorAddHdmaTable
	lda		#$443E 	
	sta.l		colorAddHdmaTable+2
	lda		#$4010
	sta.l		colorAddHdmaTable+4
	lda		#$4402
	sta.l		colorAddHdmaTable+6
	lda		#0			; End of table
	sta.l		colorAddHdmaTable+8
	
	; HOFS/VOFS hdma table in RAM
	lda		#78
	sta.l		scrollHdmaTable
	lda		#0
	sta.l		scrollHdmaTable+1	; hofs = 0
	sta.l		scrollHdmaTable+2
	sta.l		scrollHdmaTable+3	; vofs = 0
	sta.l		scrollHdmaTable+4
	lda		#122			; the scanline counts must be <= 127, which is why it's split into two parts (78 + 122)
	sta.l		scrollHdmaTable+5
	lda		#0
	sta.l		scrollHdmaTable+6	; hofs = 0
	sta.l		scrollHdmaTable+7
	sta.l		scrollHdmaTable+8	; vofs = 0
	sta.l		scrollHdmaTable+9
	lda		#2
	sta.l		scrollHdmaTable+10
	lda		#0
	sta.l		scrollHdmaTable+11
	sta.l		scrollHdmaTable+12
	sta.l		scrollHdmaTable+13
	sta.l		scrollHdmaTable+14
	lda		#0
	sta.l		scrollHdmaTable+15

	stz		updateSongList
	stz		longTitleOffs
	stz		longTitleOffs+1
	lda		#1
	sta		updateLongTitle

	lda		#24
	sta		highlighted
	jsr		MoveToNextSong
	jsr		PutsLongTitle
	
	jsr		LoadVgm
 
 	jsr		UpdateOAM
 	
 	lda     	#$0F  		; End VBlank, setting brightness to 15 (100%).
 	sta     	REG_DISPCNT
 	lda		#1
 	sta		REG_OBSEL	; OBJ chr base = 0x4000

	; Loop forever.
Forever:
	sep		#$20
	lda		updateSongList
	beq		+
	stz		updateSongList
	jsr		PutsSongs
+:
	sep		#$20
	lda		updateLongTitle
	beq		+
	stz		updateLongTitle
	jsr		PutsLongTitle
+:	
	jmp Forever
 
   

; ################################################################################################


	
 VBlank:
 	php
 	rep		#$20
 	pha
 	phx
 	sep		#$20
 	
 	; Read joypad1, NES-style
 	lda		#1
 	sta		$4016
	stz		joyData
	stz		joyData+1
 	lda		#0
 	sta		$4016
	ldx		#12
-:
	lda		$4016
	lsr		a
	rep		#$20
	rol		joyData
	sep		#$20
	dex
	bne		-
	
	rep		#$20
	lda		joyData
	tax
	eor		joyMask
	stx		joyMask
	and		joyMask
	sta		joyData
	sep		#$20
	lda		joyMask+1
	and		#$D
	sta		joyMask+1	; Don't block Select being held
	
 	lda		ignoreLoops
 	bne		+
 	lda		REG_APUI00
 	cmp		spcMirrorVal	; Has the song looped?
 	beq		+
 	jsr		StopVgm
 	jsr		UpdateVolumeBars
 	lda		highlighted
 	cmp		playing
 	bne		load_new_song
 	lda		highlighted
 	cmp		#NUM_SONGS-1
 	bne		not_at_last_song
 	stz		highlighted
 	stz		nextSong
 	dec		nextSong
 	stz		firstShown
 	lda		#1
 	sta		updateSongList
 	bra		at_next_song
 not_at_last_song:	
 	jsr		MoveToNextSong
 at_next_song:
 	lda		highlighted
 	sec
 	sbc		firstShown
 	asl		a
 	asl		a
 	asl		a
 	asl		a
 	ina
	sta.l		colorAddHdmaTable
load_new_song:
 	lda		nextSong
 	ina
 	;and		#31
 	cmp		#NUM_SONGS
 	bcc		no_song_wrap
 	sec
 	sbc		#NUM_SONGS
 no_song_wrap:
 	sta		nextSong
 	jsr		LoadVgm
 	jmp		vblank_processing_done 

 +:

	lda		joyData
	and		#$10		; Right
	beq		+
	jsr		NextFilter
	jsr		UpdateOAM
	jmp		vblank_processing_done
+:


	lda		joyData
	and		#$40		; Down
	;beq		+
	bne		down_pressed
	bit		joyMask
	bvc		+
	dec		downRepeatDelay
	bne		+
	lda		#KEY_REPEAT_SHORT_DELAY
	sta		downRepeatDelay
	bra		down_held
down_pressed:	
	lda		#KEY_REPEAT_INITIAL_DELAY
	sta		downRepeatDelay
down_held:
	jsr		MoveToNextSong
	sep		#$20
 	lda		highlighted
 	sec
 	sbc		firstShown
 	asl		a
 	asl		a
 	asl		a
 	asl		a
 	ina
	sta.l		colorAddHdmaTable
+:

	lda		joyData
	and		#$80		; Up
	;beq		+
	bne		up_pressed
	bit		joyMask
	bpl		+
	dec		upRepeatDelay
	bne		+
	lda		#KEY_REPEAT_SHORT_DELAY
	sta		upRepeatDelay
	bra		up_held
up_pressed:	
	lda		#KEY_REPEAT_INITIAL_DELAY
	sta		upRepeatDelay
up_held:
	jsr		MoveToPrevSong
	sep		#$20
 	lda		highlighted
 	sec
 	sbc		firstShown
 	asl		a
 	asl		a
 	asl		a
 	asl		a
 	ina
	sta.l		colorAddHdmaTable
+:

	;==== B ====
	lda		joyData+1
	and		#$08		
	beq		+
	lda		playing
	cmp		highlighted
	beq		pause_song
	jsr		StopVgm
 	jsr		UpdateVolumeBars
 	lda		highlighted
 	sta		nextSong
 	jsr		LoadVgm
 	jmp		vblank_processing_done
pause_song:
	jsr		PauseVgm
	jmp		vblank_processing_done	
+:

	;==== A ====
	lda		joyData
	and		#$08		
	beq		+
	jsr		RestartVgm
+:

	;==== Start ====
	lda		joyData+1
	and		#$01		
	beq		+
	jsr		ToggleLooping
+:

	;==== R ====
	lda		joyData
	and		#$01		
	beq		+
	lda		volumeBars
	ina
	cmp		#9
	beq		+
	sta		volumeBars
	jsr		SetMvol
	jsr		UpdateVolumeBars
	jmp		vblank_processing_done
+:

	;==== L ====
	lda		joyData
	and		#$02		
	beq		+
	lda		volumeBars
	dea
	bmi		+
	sta		volumeBars
	jsr		SetMvol
	jsr		UpdateVolumeBars
	jmp		vblank_processing_done
+:

	;==== Y ====
	sep		#$20
	lda.b		joyData+1
	and		#4			
	beq		+
	lda		joyData+1
	and		#$02		; Select held?
	bne		SelY
	lda		#1
	jsr		ToggleChannel
	jmp		vblank_processing_done
SelY:
	lda		#3
	jsr		ToggleChannel
	jmp		vblank_processing_done
+:

	;==== X ====
	lda		joyData
	and		#$04		
	beq		+
	lda		joyData+1
	and		#$02		
	bne		SelX
	lda		#0
	jsr		ToggleChannel
	jmp		vblank_processing_done
SelX:
	lda		#2
	jsr		ToggleChannel
	jmp		vblank_processing_done
+:


 	jsr		UpdateVolumeBars

	lda		songListUpdated
	beq		+
	stz		songListUpdated
	rep 		#$30
	pea		1088 ;24
	pea		$0D00			; BG2 is at $C00
	pea		$007E			; bank(songListBuffer)
	pea		$8000			; offset(songListBuffer)
	jsl		LoadVram
	tsc	
	clc
	adc		#8
	tcs
	sep		#$20
+:

	lda		longTitleUpdated
	beq		+
	stz		longTitleUpdated
	rep 		#$30
	pea		58				; 29 chars * 2 bytes
	sep		#$20
	lda		#0
	xba
	lda		highlighted
	sec
	sbc		firstShown
	rep		#$20
	asl		a
	asl		a
	asl		a
	asl		a
	asl		a
	asl		a
	clc
	adc		#$D21			; VRAM word address = $0D21 + (highlighted-firstShown)*64
	pha
	pea		$007E			; bank(longTitleBuffer)
	pea		$6000			; offset(longTitleBuffer)
	jsl		LoadVram
	tsc	
	clc
	adc		#8
	tcs
	sep		#$20
+:

	jsr		HdmaChn1
	; Update the HOFS/VOFS values for the scrolltext BG
	rep		#$20
	lda.l		scrollHdmaTable+6	; hofs
	ina
	ina
	sta.l		scrollHdmaTable+6
	cmp		#$100
	bne		+
	sep		#$20
	jsr		CopyScroller
	jsr		PutsScroller
+:
	jsr		HdmaChn2
	jsr		HdmaChn3
	sep		#$20
	lda		#0
	xba
	lda.l		scrollHdmaTable+6
	tax
	lda.l		bigscroll_sinus,x
	eor		#$ff
	sta.l		scrollHdmaTable+8	; vofs
	
	lda		#$E			; Enable HDMA channels 1,2,3
	sta.l		REG_HDMAEN

	; Update the time string once a second
	lda		paused
	bne		+
	dec		secDelay
	bne		+
	lda		framesPerSec
	sta		secDelay
	lda		playbackTimeString+4
	ina
	sta		playbackTimeString+4
	cmp		#':'
	bne		time_string_ok
	lda		#'0'
	sta		playbackTimeString+4
	lda		playbackTimeString+3
	ina
	sta		playbackTimeString+3
	cmp		#'6'
	bne		time_string_ok
	lda		#'0'
	sta		playbackTimeString+3
	lda		playbackTimeString+1
	ina
	sta		playbackTimeString+1
	cmp		#':'
	bne		time_string_ok
	lda		#'0'
	sta		playbackTimeString+1	
	lda		playbackTimeString
	ina
	sta		playbackTimeString
	cmp		#':'
	bne		time_string_ok
	lda		#'0'
	sta		playbackTimeString
time_string_ok:
	jsr		PutsTime
+:	
	
	sep		#$20
	dec		scrollDelay1
	bne		++++
	lda		framesPerSec
	clc
	adc		framesPerSec
	adc		framesPerSec
	sta		scrollDelay1
	lda		titleOffs
	clc
	adc		#22
	cmp		titleLen2
	bcs		+
	sta		titleOffs
	bra		+++
+:
	stz		titleOffs
+++:
	jsr		PutsTitle

	lda		authorOffs
	clc
	adc		#22
	cmp		authorLen2
	bcs		+
	sta		authorOffs
	bra		+++
+:
	stz		authorOffs
+++:
	jsr		PutsAuthor

++++:
	sep		#$20
	dec		scrollDelay2
	bne		vblank_processing_done
	lda		framesPerSec
	clc
	adc		framesPerSec
	adc		framesPerSec
	sta		scrollDelay2
	lda		longTitleOffs
	clc
	adc		#29
	cmp		longTitleLen
	bcs		gone_past_song_title
	sta		longTitleOffs
	stz		longTitleOffs+1
	bra		long_title_updated
gone_past_song_title:
	stz		longTitleOffs
	stz		longTitleOffs+1
long_title_updated:
	lda		#1
	sta		updateLongTitle
	
vblank_processing_done: 
 	rep		#$20
 	plx
   	pla
   	plp
   	rti


; ################################################################################################


pressed_play_button:
	.dw $00A1		; BG address (BG1 is at $000)
	.dw 15,16,17		; top tiles	
	.dw 34,35,29		; bottom tiles

depressed_play_button:
	.dw $00A1
	.dw 8,9,10
	.dw 27,28,29

pressed_pause_button:
	.dw $00A4
	.dw 18,19,17
	.dw 36,37,29

depressed_pause_button:
	.dw $00A4
	.dw 11,12,10
	.dw 30,31,29

pressed_repeat_button:
	.dw $00A7
	.dw 20,21,17
	.dw 38,39,29

depressed_repeat_button:
	.dw $00A7
	.dw 13,14,10
	.dw 32,33,29

palette:
 .incbin "gfx\logo2.pal"
palette_end:
.incbin "gfx\logo_top.pal"

map:
 .incbin "gfx\logo.nam"
map_end:
.incbin "gfx\logo_top.nam"


.include "aplib_decrunch.asm"


; The VGM playback driver for the SPC
player:
.incbin "vgmplay.bin"

; Font (8x8) patterns
font2bpp: 
  .incbin "gfx\adore2bpp.pat"
font4bpp: 
  .incbin "gfx\adore4bpp.pat"

; Volume histogram patterns
volume_bars:
 .incbin "gfx\volume.pat"
volume_pal:
 .incbin "gfx\volume.pal"

; Big (16x16) font patterns
bigfont:
.incbin "gfx\font5.pat"
bigfont_colors:
.incbin "gfx\font5.pal"

font4bpp_colors:
.dw 0,$4A48

font2bpp_colors:
.dw $4400,$5AD8,$5C21 
font2bpp_colors2:
.dw $5C21,$5AD8,$4400

filter_buttons:
 .incbin "gfx\filter_buttons.chr"


tilesize_hdma_table:
.db 78, $01 
.db 122, $11
.db 2, $01
.db 0

filter_buttons_colors:
 .incbin "gfx\filter_buttons.pal"

game_string1:  .db "Alex Kidd in Miracle World - ",0
game_string2:  .db "Alien 3 - ",0
game_string3:  .db "Back to the Future II - ",0
game_string4:  .db "Batman Returns - ",0
game_string5:  .db "Battletoads - ",0
game_string6:  .db "Blade Eagle 3-D - ",0
game_string7:  .db "Chuck Rock - ",0
game_string8:  .db "Cosmic Spacehead - ",0
game_string9:  .db "Fantasy Zone II - ",0
game_string10: .db "Gunstar Heroes - ",0
game_string11: .db "Marble Madness - ",0
game_string12: .db "Ninja Gaiden - ",0
game_string13: .db "Power Strike II - ",0
game_string14: .db "Ristar - ",0
game_string15: .db "Smurfs - ",0
game_string16: .db "Sonic the Hedgehog - ",0
game_string17: .db "Sonic the Hedgehog 2 - ",0
game_string18: .db "Tintin in Tibet - ",0
game_string19: .db "Wonder Boy III : The Dragon's Trap - ",0
game_string20: .db "Y's : The Vanished Omens - ",0
game_string21: .db "After Burner - ",0
game_stringNA: .db "N/A - ",0

title_string1:  .db "Main Theme",0
title_string2:  .db "Underwater",0
title_string3:  .db "Title Screen",0
title_string4:  .db "Episode 2",0
title_string5:  .db "Stage 1",0
title_string6:  .db "Ending",0
title_string7:  .db "Turbo Tunnel (Bike)",0
title_string8:  .db "Legend of the Gunstars",0
title_string9:  .db "Military on the Max-Power",0
title_string10: .db "Empire : The Final Assault",0
title_string11: .db "Pursuit in Tokyo",0
title_string12: .db "Story",0
title_string13: .db "Inside the Volcano",0
title_string14: .db "Green Hill Zone",0
title_string15: .db "Bridge Zone",0
title_string16: .db "Underground Zone",0
title_string17: .db "Upper Mountain",0
title_string18: .db "Mountain Top",0
title_string19: .db "Cave",0
title_string20: .db "Side-Crawler's Dance",0
title_string21: .db "Holders of Power",0
title_string22: .db "Palace of Destruction",0
title_string23: .db "Final Battle",0
title_string24: .db "Theme of Adol",0
title_string25: .db "Silly Race",0
title_string26: .db "Du-Di-Da!",0
title_string27: .db "The Flight on a Stork",0
title_string28: .db "Yeti's Cave",0
title_string29: .db "1Ultra",0
title_string30: .db "Above Slumberland",0
title_string31: .db "En Vard Fyra Javel",0
title_string32: .db "Hexaprism Hall",0
title_string33: .db "Let's Kill The Rainforest",0
title_string34: .db "SnoferApostrophySInfiltrationMission",0
title_string35: .db "Magnus",0
title_string36: .db "My New Used Car",0
title_string37: .db "Run Under Fire",0
title_string38: .db "That Other Final Conflict",0
title_string39: .db "Turn Off Your Radio",0
title_string40: .db "After Burner",0
title_string41: .db "Magic Beans",0

author_string1:  .db "Tokuhiko Uwabo",0
author_string2:  .db "Matt Furniss",0
author_string3:  .db "Unknown",0
author_string4:  .db "M Kawashima, M Ichikawa, Yuzo Koshiro",0
author_string5:  .db "David Wise",0
author_string6:  .db "Matt Furniss, Matthew Simmonds",0
author_string7:  .db "Allister Brimble, Lyndon Sharp",0
author_string8:  .db "K Hanzawa, H Sakimoto, M Yuzuno, M Yoshida, Y Mizusawa",0
author_string9:  .db "Fumi",0
author_string10: .db "PSG Factory",0
author_string11: .db "Alberto Jose Gonzalez",0
author_string12: .db "Masato Nakamura, Yuzo Koshiro",0
author_string13: .db "Yuzo Koshiro",0
author_string14: .db "Gatao, Nao Chan, Dawasa",0
author_string15: .db "Shinichi Sakamoto",0
author_string16: .db "Yuzo Koshiro, Sound Team JDK, Tokuhiko Uwabo,"
                 .db "Izuho Takeuchi, Neko",0
author_string17: .db "Brad Fuller, Hal Canon, Matt Furniss", 0
author_string18: .db "LKJ.Chikayo, Tomoko Sasaki", 0
author_string19: .db "Chris Kelly", 0
author_string20: .db "Tadpole", 0
author_string21: .db "AbbeAbyss", 0
author_string22: .db "Kulor", 0
author_string23: .db "RushJet1", 0
author_string24: .db "ne7", 0
author_string25: .db "Shiru", 0
author_string26: .db "thematrixeatsyou", 0
author_string27: .db "Hiroshi Miyauchi, Tokuhiko Uwabo", 0
gauthor_string19: .db "Chris Kelly - ", 0
gauthor_string20: .db "Tadpole - ", 0
gauthor_string21: .db "AbbeAbyss - ", 0
gauthor_string22: .db "Kulor - ", 0
gauthor_string23: .db "RushJet1 - ", 0
gauthor_string24: .db "ne7 - ", 0
gauthor_string25: .db "Shiru - ", 0
gauthor_string26: .db "thematrixeatsyou - ", 0
spaces: .db "                            ",0

string_table:
.dw game_string21, title_string40, author_string27, 0
.dw game_string1, title_string1, author_string1, 0	; 0
.dw game_string1, title_string2, author_string1, 0
.dw game_string2, title_string3, author_string2, 0
.dw game_string2, title_string4, author_string2, 0
.dw game_string3, title_string3, author_string3, 0
.dw game_string4, title_string5, author_string4, 0
.dw game_string4, title_string6, author_string4, 0
.dw game_string5, title_string1, author_string5, 0
.dw game_string5, title_string7, author_string5, 0	; 8
.dw game_string6, title_string6, author_string3, 0
.dw game_string7, title_string3, author_string6, 0
.dw game_string8, title_string3, author_string7, 0
.dw game_string9, title_string3, author_string3, 0
.dw game_string10, title_string8, author_string8, 0
.dw game_string10, title_string9, author_string8, 0
.dw game_string10, title_string10, author_string8, 0
.dw game_string11, title_string25, author_string17, 0	; 16
.dw game_string12, title_string3, author_string9, 0
.dw game_string12, title_string11, author_string9, 0
.dw game_string13, title_string12, author_string10, 0
.dw game_string13, title_string5, author_string10, 0
.dw game_string13, title_string6, author_string10, 0
.dw game_string14, title_string26, author_string18, 0
.dw game_string15, title_string13, author_string11, 0
.dw game_string15, title_string27, author_string11, 0
.dw game_string16, title_string14, author_string12, 0
.dw game_string16, title_string15, author_string13, 0
.dw game_string17, title_string16, author_string14, 0
.dw game_string18, title_string3, author_string11, 0
.dw game_string18, title_string17, author_string11, 0
.dw game_string18, title_string18, author_string11, 0
.dw game_string18, title_string19, author_string11, 0
.dw game_string18, title_string28, author_string11, 0
.dw game_string20, title_string21, author_string16, 0
.dw game_string20, title_string22, author_string16, 0
.dw game_string20, title_string23, author_string16, 0
.dw game_string20, title_string24, author_string16, 0
.dw gauthor_string21, title_string31, author_string21, 0
.dw gauthor_string19, title_string29, author_string19, 0
.dw gauthor_string19, title_string35, author_string19, 0
.dw gauthor_string22, title_string32, author_string22, 0
.dw gauthor_string22, title_string34, author_string22, 0
.dw gauthor_string24, title_string41, author_string24, 0
.dw gauthor_string23, title_string33, author_string23, 0
.dw gauthor_string23, title_string36, author_string23, 0
.dw gauthor_string23, title_string38, author_string23, 0
.dw gauthor_string25, title_string37, author_string25, 0
.dw gauthor_string20, title_string30, author_string20, 0
.dw gauthor_string26, title_string39, author_string26, 0


; Y-offsets for the scroll-text
bigscroll_sinus:
.db $FF,$FF,$FF,$FF,$FE,$FE,$FE,$FE,$FD,$FD,$FD,$FD,$FC,$FC,$FC,$FB
.db $FB,$FB,$FB,$FA,$FA,$FA,$FA,$F9,$F9,$F9,$F9,$F8,$F8,$F8,$F8,$F7
.db $F7,$F7,$F7,$F6,$F6,$F6,$F6,$F5,$F5,$F5,$F5,$F4,$F4,$F4,$F4,$F4
.db $F3,$F3,$F3,$F3,$F2,$F2,$F2,$F2,$F2,$F1,$F1,$F1,$F1,$F1,$F0,$F0
.db $F0,$F0,$F0,$EF,$EF,$EF,$EF,$EF,$EE,$EE,$EE,$EE,$EE,$EE,$EE,$ED
.db $ED,$ED,$ED,$ED,$ED,$EC,$EC,$EC,$EC,$EC,$EC,$EC,$EC,$EB,$EB,$EB
.db $EB,$EB,$EB,$EB,$EB,$EB,$EB,$EB,$EA,$EA,$EA,$EA,$EA,$EA,$EA,$EA
.db $EA,$EA,$EA,$EA,$EA,$EA,$EA,$EA,$EA,$EA,$EA,$EA,$EA,$EA,$EA,$EA
.db $E9,$EA,$EA,$EA,$EA,$EA,$EA,$EA,$EA,$EA,$EA,$EA,$EA,$EA,$EA,$EA
.db $EA,$EA,$EA,$EA,$EA,$EA,$EA,$EA,$EA,$EB,$EB,$EB,$EB,$EB,$EB,$EB
.db $EB,$EB,$EB,$EB,$EC,$EC,$EC,$EC,$EC,$EC,$EC,$EC,$ED,$ED,$ED,$ED
.db $ED,$ED,$EE,$EE,$EE,$EE,$EE,$EE,$EE,$EF,$EF,$EF,$EF,$EF,$F0,$F0
.db $F0,$F0,$F0,$F1,$F1,$F1,$F1,$F1,$F2,$F2,$F2,$F2,$F2,$F3,$F3,$F3
.db $F3,$F4,$F4,$F4,$F4,$F4,$F5,$F5,$F5,$F5,$F6,$F6,$F6,$F6,$F7,$F7
.db $F7,$F7,$F8,$F8,$F8,$F8,$F9,$F9,$F9,$F9,$FA,$FA,$FA,$FA,$FB,$FB
.db $FB,$FB,$FC,$FC,$FC,$FD,$FD,$FD,$FD,$FE,$FE,$FE,$FE,$FF,$FF,$FF



; Pointers to the packed VGM files. The format is as follows:
;
; struct {
;   WORD vgmBytesInBank; 	// The amount of data from the VGM file in this bank
;   WORD offset;		 // Offset within the bank where the data is found	
;   WORD bank;			 // Bank where the data is found
;   WORD padding;		 // Just to get the table entries aligned on a power of 2
; }
vgm_table:
.dw 9072, song0, :song0,0, 0,0,0,0 ;*
.dw 5270, song1,:song1,0, 0,0,0,0 ;*
.dw 12650,song2,:song2,0, 0,0,0,0 ;*
.dw 18514,song3,:song3,0, 0,0,0,0 ;*
.dw 27882,song4,:song4,0, 0,0,0,0 ;*
.dw 24805,song5,:song5,0, 0,0,0,0 ;*
.dw 5389,song6,:song6,0, 0,0,0,0 ;*
.dw 12843,song7,:song7,0, 0,0,0,0 ;*
.dw 13938,song8,:song8,0, 0,0,0,0 ;*
.dw 16303,song9,:song9,0, 0,0,0,0 ;*

.dw 7166,song10,:song10,0, 0,0,0,0 ;*
.dw 25325,song11,:song11,0, 0,0,0,0 ;*
.dw 19815,song12,:song12,0, 0,0,0,0 ;*
.dw 2432,song13,:song13,0, 0,0,0,0 ;*
.dw 32768,song14lo,:song14lo,0, 14479,song14hi,:song14hi,0 ;*
.dw 28428,song15,:song15,0, 0,0,0,0 ;*
.dw 32089,song16,:song16,0, 0,0,0,0 ;*
.dw 14397,song17,:song17,0, 0,0,0,0 ;*
.dw 10282,song18,:song18,0, 0,0,0,0 ;*
.dw 13136,song19,:song19,0, 0,0,0,0 ;*

.dw 8147,song20,:song20,0, 0,0,0,0 ;*
.dw 6811,song21,:song21,0, 0,0,0,0 ;*
.dw 7298,song22,:song22,0, 0,0,0,0 ;*
.dw 4768,song23,:song23,0, 0,0,0,0 ;*
.dw 30782,song24,:song24,0, 0,0,0,0 ;*
.dw 14159,song25,:song25,0, 0,0,0,0 ;*
.dw 10822,song26,:song26,0, 0,0,0,0 ;*
.dw 4655,song27,:song27,0, 0,0,0,0 ;*
.dw 21282,song28,:song28,0, 0,0,0,0 ;*
.dw 28311,song29,:song29,0, 0,0,0,0 ;*

.dw 25822,song30,:song30,0, 0,0,0,0 ;*
.dw 30051,song31,:song31,0, 0,0,0,0 ;*
.dw 32768,song32lo,:song32lo,0, 13016,song32hi,:song32hi,0 ;*
.dw 17186,song33,:song33,0, 0,0,0,0
.dw 6932,song34,:song34,0, 0,0,0,0 ;*
.dw 13046,song35,:song35,0, 0,0,0,0 ;*
.dw 3096,song36,:song36,0, 0,0,0,0 ;*
.dw 8866,song37,:song37,0, 0,0,0,0

.dw 27089,song38,:song38,0, 0,0,0,0 ;*
.dw 32768,song39lo,:song39lo,0, 1232,song39hi,:song39hi,0 ;*
.dw 19227,song40,:song40,0, 0,0,0,0 ;*
.dw 18842,song41,:song41,0, 0,0,0,0 ;*
.dw 27425,song42,:song42,0, 0,0,0,0 ;*
.dw 26940,song43,:song43,0, 0,0,0,0 ;*
.dw 10995,song44,:song44,0, 0,0,0,0
.dw 11645,song45,:song45,0, 0,0,0,0 ;*
.dw 20757,song46,:song46,0, 0,0,0,0 ;*
.dw 13786,song47,:song47,0, 0,0,0,0 ;*
.dw 13986,song48,:song48,0, 0,0,0,0 ;*
.dw 21281,song49,:song49,0, 0,0,0,0 ;*

 .ends
 
 .bank 1
 .section "bank1data"
 patterns:
  .incbin "gfx\logo.apl"
 patterns_end:
.incbin "gfx\logo_top.apl"
song43: .incbin "vgp\song43.vgp"
.ends


; The packed VGM files. Some are split across two banks (songNNlo/songNNhi)
;
 .bank 2
 .section "musicDataLow"
.dsb 32768,0
 .ends
 .bank 3
 .section "bank3data"
 song0: .incbin "vgp\song0.vgp"
 song1: .incbin "vgp\song1.vgp"
 song17: .incbin "vgp\song17.vgp"
 .ends
 .bank 4
 .section "bank4data"
 song14lo: .incbin "vgp\song14.vgp" read $8000
 .ends
 .bank 5
 .section "bank5data"
 song29: .incbin "vgp\song29.vgp" 
 .ends
 .bank 6
 .section "bank6data"
 song41: .incbin "vgp\song41.vgp"
 song27: .incbin "vgp\song27.vgp"
 .ends
 .bank 7
 .section "bank7data"
 song30: .incbin "vgp\song30.vgp" 
 .ends
 .bank 8
 .section "bank8data"
 song42: .incbin "vgp\song42.vgp"
 .ends
 .bank 9
 .section "bank9data"
 song32hi: .incbin "vgp\song32.vgp" skip $8000
 song35: .incbin "vgp\song35.vgp"
 .ends
 .bank 10
 .section "bank10data"
 song12: .incbin "vgp\song12.vgp"
 song18: .incbin "vgp\song18.vgp"
 .ends
 .bank 11
 .section "bank11data"
 song16: .incbin "vgp\song16.vgp"
 .ends
 .bank 12
 .section "bank12data"
 song24: .incbin "vgp\song24.vgp"
 song39hi: .incbin "vgp\song39.vgp" skip $8000
 .ends
 .bank 13
 .section "bank13data"
 song31: .incbin "vgp\song31.vgp"
 .ends
 .bank 14
 .section "bank14data"
 song48: .incbin "vgp\song48.vgp" 
 song47: .incbin "vgp\song47.vgp"
 .ends
 .bank 15
 .section "bank15data"
 song2: .incbin "vgp\song2.vgp"
 song3: .incbin "vgp\song3.vgp"
 .ends
 .bank 16
 .section "bank16data"
 song11: .incbin "vgp\song11.vgp"
  song22: .incbin "vgp\song22.vgp"
 .ends
 .bank 17
 .section "bank17data"
 song6: .incbin "vgp\song6.vgp"
 song25: .incbin "vgp\song25.vgp"
 song26: .incbin "vgp\song26.vgp"
 .ends
 .bank 18
 .section "bank18data"
 song39lo: .incbin "vgp\song39.vgp" read $8000
 .ends
 .bank 19
 .section "bank19data"
 song8: .incbin "vgp\song8.vgp"
 song9: .incbin "vgp\song9.vgp"
 .ends
 .bank 20
 .section "bank20data"
 song45: .incbin "vgp\song45.vgp"
 song40: .incbin "vgp\song40.vgp"
 .ends
 .bank 21
 .section "bank21data"
 song5: .incbin "vgp\song5.vgp"
 song10: .incbin "vgp\song10.vgp"
 .ends
 .bank 22
 .section "bank22data"
 song15: .incbin "vgp\song15.vgp"
 .ends
 .bank 23
 .section "bank23data"
 song46: .incbin "vgp\song46.vgp"
 song44: .incbin "vgp\song44.vgp"
 .ends
 .bank 24
 .section "bank24data"
 song38: .incbin "vgp\song38.vgp"
 .ends
 .bank 25
 .section "bank25data"
 song7: .incbin "vgp\song7.vgp"
 song13: .incbin "vgp\song13.vgp"
 song14hi: .incbin "vgp\song14.vgp" skip $8000
 .ends

 .bank 26
 .section "bank26data"
 song49: .incbin "vgp\song49.vgp"
 song20: .incbin "vgp\song20.vgp" 
 .ends
 .bank 27
 .section "bank27data"
 song19: .incbin "vgp\song19.vgp"
 song23: .incbin "vgp\song23.vgp"  
 song36: .incbin "vgp\song36.vgp"
 song37: .incbin "vgp\song37.vgp"
 .ends
 .bank 28
 .section "bank28data"
 song21: .incbin "vgp\song21.vgp"
 song28: .incbin "vgp\song28.vgp" 
 .ends
 .bank 29
 .section "bank29data"
 song33: .incbin "vgp\song33.vgp"
 song34: .incbin "vgp\song34.vgp"
 
 ; The scroll-text string, pre-converted from ASCII to tile numbers, and with the tile attributes
 ; added (palette number).
 scrolltext:
.dw $1100,$1100,$1100,$1100,$1100,$1100,$1100,$1100,$1100,$1100,$1124,$1100,$11C8,$11A0,$118A,$1100
.dw $114E,$1140,$1140,$1100,$1186,$11A8,$11CA,$1184,$1100,$1164,$1100,$1144,$11AC,$1188,$1100,$118A
.dw $1188,$11A2,$11C8,$11A2,$11AE,$11AC,$1100,$1124,$1100,$1100,$1100,$1100,$1100,$1100,$1100,$1100
.dw $1100,$1100,$1100,$1100,$1100,$1100,$1100,$1100,$11C8,$11A0,$118A,$1100,$11C6,$11AC,$118A,$11C6
.dw $1100,$11C6,$11A2,$11AC,$118E,$11C6,$1100,$1182,$118E,$1182,$11A2,$11AC,$1100,$11A2,$11AC,$1100
.dw $11C8,$11A0,$118A,$1100,$11E2,$118A,$1182,$11C4,$1100,$1144,$1140,$1142,$1144,$1100,$1100,$1100
.dw $1100,$1100,$1100,$1100,$1186,$11AE,$11AC,$11C8,$11C4,$11AE,$11A8,$11C6,$1164,$1100,$11E6,$11CA
.dw $11C0,$11EA,$112E,$11E6,$1188,$11AE,$11CE,$11AC,$11EA,$1100,$11C6,$1186,$11C4,$11AE,$11A8,$11A8
.dw $11C6,$1100,$11A2,$11AC,$1100,$11C8,$11A0,$118A,$1100,$11C6,$11AE,$11AC,$118E,$11C6,$1100,$11A8
.dw $11A2,$11C6,$11C8,$112C,$1100,$11C8,$11A0,$118A,$11C4,$118A,$110E,$11C6,$1100,$1182,$1100,$11C8
.dw $11AE,$11C8,$1182,$11A8,$1100,$11AE,$118C,$1100,$114A,$1140,$1100,$11C6,$11AE,$11AC,$118E,$11C6
.dw $1100,$11C8,$11AE,$1100,$1186,$11A0,$11AE,$11AE,$11C6,$118A,$1100,$118C,$11C4,$11AE,$11AA,$1100
.dw $1120,$1182,$11C0,$11C0,$11C4,$11AE,$11E0,$11A2,$11AA,$1182,$11C8,$118A,$11A8,$11E2,$1100,$1142
.dw $112C,$114A,$1100,$11A0,$11AE,$11CA,$11C4,$1100,$11AE,$118C,$1100,$11AA,$11CA,$11C6,$11A2,$1186
.dw $1122,$112C,$1100,$1100,$11E6,$1184,$11EA,$1100,$11C0,$11A8,$1182,$11E2,$11C6,$112E,$11C0,$1182
.dw $11CA,$11C6,$118A,$11C6,$1100,$11C8,$11A0,$118A,$1100,$1186,$11CA,$11C4,$11C4,$118A,$11AC,$11C8
.dw $11A8,$11E2,$1100,$11A0,$11A2,$118E,$11A0,$11A8,$11A2,$118E,$11A0,$11C8,$118A,$1188,$1100,$11C6
.dw $11AE,$11AC,$118E,$112C,$1100,$1100,$11E6,$1182,$11EA,$1100,$11C4,$118A,$11C6,$11C8,$1182,$11C4
.dw $11C8,$11C6,$1100,$11C8,$11A0,$118A,$1100,$1186,$11CA,$11C4,$11C4,$118A,$11AC,$11C8,$11A8,$11E2
.dw $1100,$11C0,$11A8,$1182,$11E2,$11A2,$11AC,$118E,$1100,$11C6,$11AE,$11AC,$118E,$112C,$1100,$1100
.dw $11E6,$11C6,$11C8,$1182,$11C4,$11C8,$11EA,$1100,$11C8,$11AE,$118E,$118E,$11A8,$118A,$11C6,$1100
.dw $11C6,$11AE,$11AC,$118E,$1100,$11A8,$11AE,$11AE,$11C0,$11A2,$11AC,$118E,$1100,$11AE,$11AC,$112E
.dw $11AE,$118C,$118C,$112C,$1100,$1100,$11E6,$11A8,$11EA,$112E,$11E6,$11C4,$11EA,$1100,$1186,$11AE
.dw $11AC,$11C8,$11C4,$11AE,$11A8,$11C6,$1100,$11C8,$11A0,$118A,$1100,$11AA,$1182,$11C6,$11C8,$118A
.dw $11C4,$1100,$11CC,$11AE,$11A8,$11CA,$11AA,$118A,$112C,$1100,$1100,$11E6,$11E0,$11EA,$1100,$11C8
.dw $11AE,$118E,$118E,$11A8,$118A,$11C6,$1100,$11C8,$11AE,$11AC,$118A,$1140,$1100,$11AE,$11AC,$112E
.dw $11AE,$118C,$118C,$112C,$1100,$1100,$11E6,$11E2,$11EA,$1100,$11C8,$11AE,$118E,$118E,$11A8,$118A
.dw $11C6,$1100,$11C8,$11AE,$11AC,$118A,$1142,$1100,$11AE,$11AC,$112E,$11AE,$118C,$118C,$112C,$1100
.dw $1100,$11E6,$11C6,$118A,$11A8,$118A,$1186,$11C8,$11EA,$1126,$11E6,$11E0,$11EA,$1100,$11C8,$11AE
.dw $118E,$118E,$11A8,$118A,$11C6,$1100,$11C8,$11AE,$11AC,$118A,$1144,$1100,$11AE,$11AC,$112E,$11AE
.dw $118C,$118C,$112C,$1100,$1100,$11E6,$11C6,$118A,$11A8,$118A,$1186,$11C8,$11EA,$1126,$11E6,$11E2
.dw $11EA,$1100,$11C8,$11AE,$118E,$118E,$11A8,$118A,$11C6,$1100,$11AC,$11AE,$11A2,$11C6,$118A,$1100
.dw $11AE,$11AC,$112E,$11AE,$118C,$118C,$112C,$1100,$1100,$11E6,$11C4,$11A2,$118E,$11A0,$11C8,$11EA
.dw $1100,$11C6,$11CE,$11A2,$11C8,$1186,$11A0,$118A,$11C6,$1100,$11C8,$11A0,$118A,$1100,$1186,$11CA
.dw $11C4,$11C4,$118A,$11AC,$11C8,$1100,$1182,$11CA,$1188,$11A2,$11AE,$1100,$118C,$11A2,$11A8,$11C8
.dw $118A,$11C4,$112C,$1100,$1100,$1100,$1100,$1100,$1100,$1100,$1100,$1100,$1100,$1100,$1100,$1100
.dw $1100,$1100,$1100,$1100,$1100,$1100,$1100,$1100,$1100,$11AA,$11AE,$11C6,$11C8,$1100,$11AE,$118C
.dw $1100,$11C8,$11A0,$118A,$1100,$11E4,$11A2,$11A6,$11C6,$1100,$1182,$11C4,$118A,$1100,$11C8,$1182
.dw $11A6,$118A,$11AC,$1100,$118C,$11C4,$11AE,$11AA,$1100,$11C6,$118A,$118E,$1182,$1100,$11AA,$1182
.dw $11C6,$11C8,$118A,$11C4,$11C6,$11E2,$11C6,$11C8,$118A,$11AA,$1100,$112E,$1100,$118E,$1182,$11AA
.dw $118A,$118E,$118A,$1182,$11C4,$1100,$118E,$1182,$11AA,$118A,$11C6,$1100,$1182,$11AC,$1188,$1100
.dw $1182,$11C4,$118A,$1100,$11C0,$11A8,$1182,$11E2,$118A,$1188,$1100,$1184,$1182,$1186,$11A6,$1100
.dw $11CA,$11C6,$11A2,$11AC,$118E,$1100,$11AA,$11E2,$1100,$11AE,$11CE,$11AC,$1100,$11CC,$118E,$11AA
.dw $1100,$1188,$11C4,$11A2,$11CC,$118A,$11C4,$1100,$118C,$11AE,$11C4,$1100,$11C8,$11A0,$118A,$1100
.dw $11C6,$11C0,$1186,$112C,$1100,$1100,$1100,$1100,$1100,$1100,$1100,$1100,$1100,$1100,$1100,$1100
.dw $1100,$1100,$1100,$1100,$1100,$1100,$1100,$1100,$1100,$1100,$11C6,$11AE,$11AA,$118A,$1100,$11C8
.dw $118A,$1186,$11A0,$11AC,$11A2,$1186,$1182,$11A8,$1100,$1188,$118A,$11C8,$1182,$11A2,$11A8,$11C6
.dw $1164,$1100,$11CE,$11A0,$118A,$11AC,$1100,$11C8,$11A0,$118A,$1100,$11C4,$11AE,$11AA,$1100,$11C6
.dw $11C8,$1182,$11C4,$11C8,$11C6,$1100,$11A2,$11C8,$1100,$11C6,$118A,$11AC,$1188,$11C6,$1100,$11C8
.dw $11A0,$118A,$1100,$11C0,$11A8,$1182,$11E2,$1184,$1182,$1186,$11A6,$1100,$1188,$11C4,$11A2,$11CC
.dw $118A,$11C4,$1100,$11C8,$11AE,$1100,$11C8,$11A0,$118A,$1100,$11C6,$11C0,$1186,$1128,$1100,$118C
.dw $11AE,$11A8,$11A8,$11AE,$11CE,$118A,$1188,$1100,$1184,$11E2,$1100,$11C8,$11A0,$118A,$1100,$11CC
.dw $118E,$11AA,$1100,$1188,$1182,$11C8,$1182,$1100,$118C,$11AE,$11C4,$1100,$11C8,$11A0,$118A,$1100
.dw $11C6,$11AE,$11AC,$118E,$1100,$11A2,$11C8,$1100,$1188,$118A,$1186,$11A2,$1188,$118A,$11C6,$1100
.dw $11C8,$11AE,$1100,$11A8,$11AE,$1182,$1188,$1100,$118C,$11A2,$11C4,$11C6,$11C8,$112C,$1100,$11A2
.dw $11C8,$1100,$11C8,$11A0,$118A,$11AC,$1100,$11C6,$118A,$11AC,$1188,$11C6,$1100,$1182,$1100,$1104
.dw $11C0,$11A8,$1182,$11E2,$1104,$1100,$1186,$11AE,$11AA,$11AA,$1182,$11AC,$1188,$1100,$11AE,$11CC
.dw $118A,$11C4,$1100,$11AE,$11AC,$118A,$1100,$11AE,$118C,$1100,$11C8,$11A0,$118A,$1100,$1186,$11AE
.dw $11AA,$11AA,$11CA,$11AC,$11A2,$1186,$1182,$11C8,$11A2,$11AE,$11AC,$1100,$11C0,$11AE,$11C4,$11C8
.dw $11C6,$1128,$1100,$11CE,$11A0,$11A2,$1186,$11A0,$1100,$11C8,$11A0,$118A,$1100,$1188,$11C4,$11A2
.dw $11CC,$118A,$11C4,$1100,$11C0,$11A2,$1186,$11A6,$11C6,$1100,$11CA,$11C0,$1100,$1182,$11AC,$1188
.dw $1100,$11C6,$11C8,$1182,$11C4,$11C8,$11C6,$1100,$11C0,$11A8,$1182,$11E2,$11A2,$11AC,$118E,$1100
.dw $11C8,$11A0,$118A,$1100,$11CC,$118E,$11AA,$1100,$1188,$1182,$11C8,$1182,$1100,$11C8,$11A0,$1182
.dw $11C8,$1100,$11A0,$1182,$11C6,$1100,$1184,$118A,$118A,$11AC,$1100,$1186,$11AE,$11C0,$11A2,$118A
.dw $1188,$1100,$11C8,$11AE,$1100,$11C6,$11C0,$1186,$1100,$11C4,$1182,$11AA,$112C,$1100,$11A2,$118C
.dw $1100,$11E2,$11AE,$11CA,$1100,$11C0,$11C4,$118A,$11C6,$11C6,$1100,$11E6,$1184,$11EA,$1100,$11C8
.dw $11AE,$1100,$11C6,$11C8,$1182,$11C4,$11C8,$1100,$11C0,$11A8,$1182,$11E2,$11A2,$11AC,$118E,$1100
.dw $1182,$11AC,$11AE,$11C8,$11A0,$118A,$11C4,$1100,$11C6,$11AE,$11AC,$118E,$1128,$1100,$1182,$1100
.dw $1186,$11AE,$11AA,$11AA,$1182,$11AC,$1188,$1100,$11CE,$11A2,$11A8,$11A8,$1100,$1184,$118A,$1100
.dw $11C6,$118A,$11AC,$11C8,$1100,$11C8,$11AE,$1100,$11C8,$11A0,$118A,$1100,$1188,$11C4,$11A2,$11CC
.dw $118A,$11C4,$1100,$11C8,$11AE,$1100,$11C8,$118A,$11A8,$11A8,$1100,$11A2,$11C8,$1100,$11C8,$11AE
.dw $1100,$11C6,$11C8,$11AE,$11C0,$1100,$11C0,$11A8,$1182,$11E2,$1184,$1182,$1186,$11A6,$1100,$1182
.dw $11AC,$1188,$1100,$118A,$11E0,$11C0,$118A,$1186,$11C8,$1100,$11C8,$11AE,$1100,$11C4,$118A,$1186
.dw $118A,$11A2,$11CC,$118A,$1100,$11AA,$11AE,$11C4,$118A,$1100,$1188,$1182,$11C8,$1182,$112C,$1100
.dw $11C8,$11A0,$118A,$1100,$11AC,$118A,$11CE,$1100,$11CC,$118E,$11AA,$1100,$11A2,$11C6,$1100,$11C8
.dw $11A0,$118A,$11AC,$1100,$11C6,$118A,$11AC,$11C8,$1100,$11C8,$11AE,$1100,$11C6,$11C0,$1186,$1100
.dw $11C4,$1182,$11AA,$1128,$1100,$118C,$11AE,$11A8,$11A8,$11AE,$11CE,$118A,$1188,$1100,$1184,$11E2
.dw $1100,$1182,$1100,$1104,$11C0,$11A8,$1182,$11E2,$1104,$1100,$1186,$11AE,$11AA,$11AA,$1182,$11AC
.dw $1188,$112C,$1100,$1100,$1182,$11A8,$11A8,$1100,$11CE,$11C4,$11A2,$11C8,$118A,$11C6,$1100,$11C8
.dw $11AE,$1100,$11C8,$11A0,$118A,$1100,$11C6,$11AC,$114E,$114C,$1148,$1160,$1162,$1100,$11A2,$11AC
.dw $1100,$11C8,$11A0,$118A,$1100,$11CC,$118E,$11AA,$1100,$1182,$11C4,$118A,$1100,$118A,$11AA,$11CA
.dw $11A8,$1182,$11C8,$118A,$1188,$1100,$11CE,$11A2,$11C8,$11A0,$1100,$11C8,$11A0,$118A,$1100,$11A0
.dw $118A,$11A8,$11C0,$1100,$11AE,$118C,$1100,$11C8,$11A0,$118A,$1100,$11C6,$112A,$1188,$11C6,$11C0
.dw $1128,$1100,$11CE,$11A0,$11A2,$11A8,$118A,$1100,$1188,$118A,$11A8,$1182,$11E2,$1100,$1186,$11AE
.dw $11AA,$11AA,$1182,$11AC,$1188,$11C6,$1100,$1182,$11C4,$118A,$1100,$11A0,$1182,$11AC,$1188,$11A8
.dw $118A,$1188,$1100,$11CA,$11C6,$11A2,$11AC,$118E,$1100,$11C8,$11A2,$11AA,$118A,$1188,$1100,$11A8
.dw $11AE,$11AE,$11C0,$11C6,$112C,$1100,$11C6,$11A2,$11AC,$1186,$118A,$1100,$11C6,$11AE,$11AA,$118A
.dw $1100,$1186,$11AE,$11AA,$11AA,$1182,$11AC,$1188,$11C6,$1100,$11C8,$1182,$11A6,$118A,$1100,$11AA
.dw $11CA,$1186,$11A0,$1100,$11A8,$11AE,$11AC,$118E,$118A,$11C4,$1100,$11C8,$11AE,$1100,$11C0,$11C4
.dw $11AE,$1186,$118A,$11C6,$11C6,$1100,$11C8,$11A0,$1182,$11AC,$1100,$11C8,$11A0,$118A,$1100,$1182
.dw $11C0,$11C0,$11C4,$11AE,$11E0,$11A2,$11AA,$1182,$11C8,$118A,$11A8,$11E2,$1100,$1144,$1146,$1100
.dw $1186,$11E2,$1186,$11A8,$118A,$11C6,$1100,$1182,$11CC,$1182,$11A2,$11A8,$1182,$1184,$11A8,$118A
.dw $1128,$1100,$1182,$1100,$1186,$11AE,$11CA,$11AC,$11C8,$118A,$11C4,$1100,$11A2,$11C6,$1100,$11CA
.dw $11C6,$118A,$1188,$1100,$11C8,$11AE,$1100,$11A6,$118A,$118A,$11C0,$1100,$11C8,$11C4,$1182,$1186
.dw $11A6,$1100,$11AE,$118C,$1100,$11C8,$11A0,$118A,$1100,$1182,$11AA,$11AE,$11CA,$11AC,$11C8,$1100
.dw $11AE,$118C,$1100,$118A,$11E0,$11C8,$11C4,$1182,$1100,$11C8,$11A2,$11AA,$118A,$1100,$11C6,$11C0
.dw $118A,$11AC,$11C8,$1128,$1100,$11CE,$11A0,$11A2,$1186,$11A0,$1100,$11A2,$11C6,$1100,$11C8,$11A0
.dw $118A,$11AC,$1100,$1188,$118A,$1188,$11CA,$1186,$11C8,$118A,$1188,$1100,$118C,$11C4,$11AE,$11AA
.dw $1100,$11C8,$11A0,$118A,$1100,$11C6,$11C0,$118A,$1186,$11A2,$118C,$11A2,$118A,$1188,$1100,$11AC
.dw $11CA,$11AA,$1184,$118A,$11C4,$1100,$11AE,$118C,$1100,$11C6,$1182,$11AA,$11C0,$11A8,$118A,$11C6
.dw $1100,$118C,$11AE,$11C4,$1100,$11C8,$11A0,$118A,$1100,$11AC,$118A,$11E0,$11C8,$1100,$1188,$118A
.dw $11A8,$1182,$11E2,$1100,$1186,$11AE,$11AA,$11AA,$1182,$11AC,$1188,$112C,$1100,$11AA,$11CA,$1186
.dw $11A0,$1100,$11AE,$118C,$1100,$11C8,$11A0,$118A,$1100,$1188,$118A,$11CC,$118A,$11A8,$11AE,$11C0
.dw $11AA,$118A,$11AC,$11C8,$1100,$11C8,$11A2,$11AA,$118A,$1100,$11AE,$11AC,$1100,$11C8,$11A0,$118A
.dw $1100,$1188,$11C4,$11A2,$11CC,$118A,$11C4,$1100,$11CE,$118A,$11AC,$11C8,$1100,$11A2,$11AC,$11C8
.dw $11AE,$1100,$11C8,$11CE,$118A,$1182,$11A6,$11A2,$11AC,$118E,$1100,$11C8,$11A0,$118A,$1100,$11C6
.dw $1182,$11AA,$11C0,$11A8,$118A,$11C6,$1128,$1100,$118C,$11C4,$118A,$11C2,$11CA,$118A,$11AC,$1186
.dw $11E2,$1100,$11C8,$1182,$1184,$11A8,$118A,$11C6,$1100,$1182,$11AC,$1188,$1100,$11C8,$11A0,$118A
.dw $1100,$11CC,$11AE,$11A8,$11CA,$11AA,$118A,$1100,$11C8,$1182,$1184,$11A8,$118A,$1100,$11C8,$11AE
.dw $1100,$118E,$118A,$11C8,$1100,$11C8,$11A0,$118A,$1100,$11AE,$11CA,$11C8,$11C0,$11CA,$11C8,$1100
.dw $11C8,$11AE,$1100,$11C6,$11AE,$11CA,$11AC,$1188,$1100,$11AA,$11AE,$11C4,$118A,$1100,$11A8,$11A2
.dw $11A6,$118A,$1100,$11C8,$11A0,$118A,$1100,$11AE,$11C4,$11A2,$118E,$11A2,$11AC,$1182,$11A8,$11C6
.dw $1166,$1100,$1182,$11C6,$1100,$11CE,$118A,$11A8,$11A8,$1100,$1182,$11C6,$1100,$11A0,$1182,$11AC
.dw $1188,$11A8,$11A2,$11AC,$118E,$1100,$11C6,$11AE,$11AA,$118A,$1100,$11C6,$11C0,$118A,$1186,$11A2
.dw $1182,$11A8,$1100,$1186,$1182,$11C6,$118A,$11C6,$1128,$1100,$11A8,$11A2,$11A6,$118A,$1100,$11C8
.dw $11A0,$118A,$1100,$11C6,$118A,$11C8,$11C8,$11A2,$11AC,$118E,$1100,$11C8,$11A0,$1182,$11C8,$1100
.dw $1182,$11A8,$11A8,$11AE,$11CE,$11C6,$1100,$11C8,$11A0,$118A,$1100,$11AC,$11AE,$11A2,$11C6,$118A
.dw $1100,$1186,$11A0,$1182,$11AC,$11AC,$118A,$11A8,$1100,$11C8,$11AE,$1100,$1184,$118A,$1100,$11CA
.dw $11C6,$118A,$1188,$1100,$1182,$11C6,$1100,$1182,$1100,$11A8,$11AE,$11CE,$112A,$118C,$11C4,$118A
.dw $11C2,$11CA,$118A,$11AC,$1186,$11E2,$1100,$11C6,$11C2,$11CA,$1182,$11C4,$118A,$1100,$11CE,$1182
.dw $11CC,$118A,$1100,$1184,$1182,$11C6,$118A,$1188,$1100,$11AE,$11AC,$1100,$11C8,$11A0,$118A,$1100
.dw $11C8,$11A0,$11A2,$11C4,$1188,$1100,$11C8,$11AE,$11AC,$118A,$1100,$1186,$11A0,$1182,$11AC,$11AC
.dw $118A,$11A8,$110E,$11C6,$1100,$11C0,$118A,$11C4,$11A2,$11AE,$1188,$112C,$1100,$1100,$11C8,$11A0
.dw $118A,$1100,$118C,$1182,$1186,$11C8,$1100,$11C8,$11A0,$1182,$11C8,$1100,$11C8,$11A0,$118A,$1100
.dw $1188,$11C4,$11A2,$11CC,$118A,$11C4,$1100,$11C4,$118A,$1182,$1188,$11C6,$1100,$11C8,$11A0,$118A
.dw $1100,$11CC,$118E,$11AA,$1100,$1188,$1182,$11C8,$1182,$1100,$11C6,$11C8,$11C4,$1182,$11A2,$118E
.dw $11A0,$11C8,$1100,$118C,$11C4,$11AE,$11AA,$1100,$11C6,$11C0,$1186,$1100,$11C4,$1182,$11AA,$1100
.dw $11AA,$118A,$1182,$11AC,$11C6,$1100,$11C8,$11A0,$1182,$11C8,$1100,$11C8,$11A0,$118A,$1100,$1188
.dw $11C4,$11A2,$11CC,$118A,$11C4,$1100,$1126,$1100,$11CC,$118E,$11AA,$1100,$11A0,$1182,$11C6,$1100
.dw $11C8,$11AE,$1100,$118C,$11A2,$11C8,$1100,$11A2,$11AC,$1100,$114C,$1148,$11A6,$1184,$112C,$1100
.dw $11C6,$11A2,$11AC,$1186,$118A,$1100,$11C8,$11A0,$118A,$1100,$1188,$11C4,$11A2,$11CC,$118A,$11C4
.dw $1100,$11AC,$118A,$118A,$1188,$11C6,$1100,$1182,$1100,$118C,$118A,$11CE,$1100,$11A6,$1100,$11C8
.dw $11A0,$11A2,$11C6,$1100,$11A8,$118A,$1182,$11CC,$118A,$11C6,$1100,$1182,$1184,$11AE,$11CA,$11C8
.dw $1100,$114A,$1160,$1100,$11A6,$1184,$1100,$11AE,$118C,$1100,$11C6,$11C0,$1182,$1186,$118A,$1100
.dw $11A8,$118A,$118C,$11C8,$1100,$118C,$11AE,$11C4,$1100,$11CC,$118E,$11AA,$1100,$1188,$1182,$11C8
.dw $1182,$112C,$1100,$1182,$11AC,$11E2,$11AE,$11AC,$118A,$1100,$118C,$1182,$11AA,$11A2,$11A8,$11A2
.dw $1182,$11C4,$1100,$11CE,$11A2,$11C8,$11A0,$1100,$11C8,$11A0,$118A,$1100,$11CC,$118E,$11AA,$1100
.dw $118C,$11AE,$11C4,$11AA,$1182,$11C8,$1100,$11C6,$11A0,$11AE,$11CA,$11A8,$1188,$1100,$11A6,$11AC
.dw $11AE,$11CE,$1100,$11C8,$11A0,$1182,$11C8,$1100,$11A2,$11C8,$1100,$11A2,$11C6,$11AC,$110E,$11C8
.dw $1100,$11CC,$118A,$11C4,$11E2,$1100,$1186,$11AE,$11AA,$11C0,$1182,$1186,$11C8,$1128,$1100,$11C6
.dw $11AE,$1100,$114A,$1160,$1100,$11A6,$1184,$1100,$11C6,$11AE,$11AA,$118A,$11C8,$11A2,$11AA,$118A
.dw $11C6,$1100,$11A4,$11CA,$11C6,$11C8,$1100,$11A2,$11C6,$11AC,$110E,$11C8,$1100,$118A,$11AC,$11AE
.dw $11CA,$118E,$11A0,$1100,$118C,$11AE,$11C4,$1100,$1182,$11AC,$1100,$11CA,$11AC,$1186,$11AE,$11AA
.dw $11C0,$11C4,$118A,$11C6,$11C6,$118A,$1188,$1100,$11CC,$118E,$11AA,$112C,$1100,$11CC,$118E,$11E4
.dw $1100,$11CE,$1182,$11C6,$1100,$11AE,$11CA,$11C8,$1100,$11AE,$118C,$1100,$11C8,$11A0,$118A,$1100
.dw $11C2,$11CA,$118A,$11C6,$11C8,$11A2,$11AE,$11AC,$1100,$11C6,$11A2,$11AC,$1186,$118A,$1100,$11C8
.dw $11A0,$118A,$1100,$1188,$118A,$1186,$11AE,$11AA,$11C0,$11C4,$118A,$11C6,$11C6,$11A2,$11AE,$11AC
.dw $1100,$11CE,$11AE,$11CA,$11A8,$1188,$1100,$1184,$118A,$1100,$118C,$1182,$11C4,$1100,$11C8,$11AE
.dw $11AE,$1100,$11C6,$11A8,$11AE,$11CE,$1100,$118C,$11AE,$11C4,$1100,$11AA,$11E2,$1100,$11C4,$118A
.dw $11C2,$11CA,$11A2,$11C4,$118A,$11AA,$118A,$11AC,$11C8,$11C6,$112C,$1100,$11CE,$11A0,$1182,$11C8
.dw $1100,$11A2,$1100,$118A,$11AC,$1188,$118A,$1188,$1100,$11CA,$11C0,$1100,$1188,$11AE,$11A2,$11AC
.dw $118E,$1100,$11CE,$1182,$11C6,$1100,$11CE,$11C4,$11A2,$11C8,$11A2,$11AC,$118E,$1100,$1182,$1100
.dw $11C8,$11AE,$11AE,$11A8,$1100,$11C8,$11A0,$1182,$11C8,$1100,$11C8,$1182,$11A6,$118A,$11C6,$1100
.dw $11CC,$118E,$11AA,$1100,$118C,$11A2,$11A8,$118A,$11C6,$1100,$1182,$11AC,$1188,$1100,$1186,$11AE
.dw $11AA,$11C0,$11C4,$118A,$11C6,$11C6,$118A,$11C6,$1100,$11C8,$11A0,$118A,$11AA,$1100,$11CA,$11C6
.dw $11A2,$11AC,$118E,$1100,$1182,$1100,$11CC,$118A,$11C4,$11E2,$1100,$11C6,$11A2,$11AA,$11C0,$11A8
.dw $118A,$1100,$11C6,$1186,$11A0,$118A,$11AA,$118A,$1100,$11A2,$1100,$1186,$1182,$11AA,$118A,$1100
.dw $11CA,$11C0,$1100,$11CE,$11A2,$11C8,$11A0,$1100,$11C8,$11A0,$1182,$11C8,$1100,$11C6,$11C8,$11C4
.dw $11A2,$11C0,$11C6,$1100,$1182,$1100,$11A8,$11AE,$11C8,$1100,$11AE,$118C,$1100,$1186,$11AE,$11AA
.dw $11AA,$1182,$11AC,$1188,$1100,$1184,$11E2,$11C8,$118A,$11C6,$1100,$1182,$11AC,$1188,$1100,$11C4
.dw $118A,$11C0,$11A8,$1182,$1186,$118A,$11C6,$1100,$11C8,$11A0,$118A,$11AA,$1100,$11CE,$11A2,$11C8
.dw $11A0,$1100,$11C6,$11A2,$11AC,$118E,$11A8,$118A,$112A,$1184,$11A2,$11C8,$1100,$118C,$11A8,$1182
.dw $118E,$11C6,$1100,$118E,$11C4,$11AE,$11CA,$11C0,$118A,$1188,$1100,$11A2,$11AC,$11C8,$11AE,$1100
.dw $1182,$1100,$1184,$11E2,$11C8,$118A,$1100,$118C,$11AE,$11C4,$1100,$118A,$11CC,$118A,$11C4,$11E2
.dw $1100,$11C6,$118A,$11CC,$118A,$11AC,$1100,$1186,$11AE,$11AA,$11AA,$1182,$11AC,$1188,$11C6,$1128
.dw $1100,$11CE,$11A2,$11C8,$11A0,$1100,$11C8,$11A0,$118A,$1100,$11AA,$11C6,$1184,$1100,$11C6,$11A2
.dw $118E,$11AC,$1182,$11A8,$11A8,$11A2,$11AC,$118E,$1100,$11CE,$11A0,$118A,$11C8,$11A0,$118A,$11C4
.dw $1100,$1182,$11A8,$11A8,$1100,$1140,$112A,$1184,$11A2,$11C8,$11C6,$1100,$11A2,$11AC,$1100,$11C8
.dw $11A0,$118A,$1100,$1186,$11CA,$11C4,$11C4,$118A,$11AC,$11C8,$1100,$118E,$11C4,$11AE,$11CA,$11C0
.dw $1100,$11C6,$11A0,$11AE,$11CA,$11A8,$1188,$1100,$1184,$118A,$1100,$11A2,$11AC,$11C8,$118A,$11C4
.dw $11C0,$11C4,$118A,$11C8,$118A,$1188,$1100,$1182,$11C6,$1100,$118C,$11C4,$1182,$11AA,$118A,$1100
.dw $1188,$118A,$11A8,$1182,$11E2,$11C6,$1100,$1120,$11CE,$11A0,$11A2,$1186,$11A0,$1100,$1182,$11C4
.dw $118A,$1100,$11C8,$11A0,$118A,$11AC,$1100,$11AE,$11AA,$11A2,$11C8,$11C8,$118A,$1188,$1100,$118C
.dw $11C4,$11AE,$11AA,$1100,$11C8,$11A0,$118A,$1100,$1186,$11AE,$11AA,$11C0,$11C4,$118A,$11C6,$11C6
.dw $118A,$1188,$1100,$11C6,$11C8,$11C4,$118A,$1182,$11AA,$1122,$112C,$1100,$11C8,$11A0,$11A2,$11C6
.dw $1100,$11CE,$1182,$11C6,$1100,$11A8,$11A2,$118E,$11A0,$11C8,$11CE,$118A,$11A2,$118E,$11A0,$11C8
.dw $1100,$118A,$11AC,$11AE,$11CA,$118E,$11A0,$1100,$11C8,$11AE,$1100,$11CA,$11AC,$11C0,$1182,$1186
.dw $11A6,$1100,$11AE,$11AC,$112A,$11C8,$11A0,$118A,$112A,$118C,$11A8,$11E2,$1128,$1100,$11CE,$11A0
.dw $11A2,$11A8,$118A,$1100,$118E,$11A2,$11CC,$11A2,$11AC,$118E,$1100,$1182,$1100,$11AA,$11CA,$1186
.dw $11A0,$112A,$11AC,$118A,$118A,$1188,$118A,$1188,$1100,$1186,$11AE,$11AA,$11C0,$11C4,$118A,$11C6
.dw $11C6,$11A2,$11AE,$11AC,$1100,$11C4,$1182,$11C8,$11A2,$11AE,$1100,$11AE,$118C,$1100,$1182,$1184
.dw $11AE,$11CA,$11C8,$1100,$1142,$112C,$1162,$1164,$1142,$1100,$1100,$1100,$1100,$1100,$1100,$1100
.dw $11C8,$11A0,$118A,$1100,$11C0,$11A8,$1182,$11E2,$118A,$11C4,$1100,$118E,$11CA,$11A2,$1100,$11CE
.dw $1182,$11C6,$1100,$1188,$11AE,$11AC,$118A,$1100,$11CA,$11C6,$11A2,$11AC,$118E,$1100,$1146,$1100
.dw $11A8,$1182,$11E2,$118A,$11C4,$11C6,$1100,$1120,$11AA,$11AE,$1188,$118A,$1100,$1142,$1122,$112C
.dw $1100,$11C8,$11A0,$11C4,$118A,$118A,$1100,$11A0,$1188,$11AA,$1182,$1100,$1186,$11A0,$1182,$11AC
.dw $11AC,$118A,$11A8,$11C6,$1100,$1182,$11C4,$118A,$1100,$11C4,$118A,$11C6,$11C0,$11AE,$11AC,$11C6
.dw $11A2,$1184,$11A8,$118A,$1100,$118C,$11AE,$11C4,$1100,$1188,$11AE,$11A2,$11AC,$118E,$1100,$11C8
.dw $11A0,$118A,$1100,$11AC,$118A,$1186,$118A,$11C6,$11C6,$1182,$11C4,$11E2,$1100,$11C8,$11A2,$11A8
.dw $118A,$1100,$11C6,$11A2,$11E4,$118A,$1100,$11C6,$11CE,$11A2,$11C8,$1186,$11A0,$1100,$1182,$11AC
.dw $1188,$1100,$11C6,$1186,$11C4,$11AE,$11A8,$11A8,$1100,$11C4,$118A,$118E,$11A2,$11C6,$11C8,$118A
.dw $11C4,$1100,$11CA,$11C0,$1188,$1182,$11C8,$118A,$11C6,$1100,$11AA,$11A2,$1188,$112A,$11C6,$1186
.dw $11C4,$118A,$118A,$11AC,$1128,$1100,$1182,$11C6,$1100,$11CE,$118A,$11A8,$11A8,$1100,$1182,$11C6
.dw $1100,$11C8,$11AE,$118E,$118E,$11A8,$11A2,$11AC,$118E,$1100,$11C8,$11A0,$118A,$1100,$1186,$11AE
.dw $11A8,$11AE,$11C4,$1100,$1182,$1188,$1188,$11A2,$11C8,$11A2,$11AE,$11AC,$1100,$118A,$118C,$118C
.dw $118A,$1186,$11C8,$1100,$118C,$11AE,$11C4,$1100,$11C8,$11A0,$118A,$1100,$11C6,$11AE,$11AC,$118E
.dw $1100,$11A8,$11A2,$11C6,$11C8,$1100,$11A0,$11A2,$118E,$11A0,$11A8,$11A2,$118E,$11A0,$11C8,$112C
.dw $1100,$1100,$1100,$1100,$1100,$1100,$1100,$1100,$1100,$1100,$1100,$1100,$1100,$1100,$11C8,$11A0
.dw $1182,$11AC,$11A6,$11C6,$1100,$11C8,$11AE,$1100,$1182,$11AC,$11AE,$11AA,$11A2,$118A,$1128,$1100
.dw $1184,$11A8,$1182,$11C4,$118E,$118E,$1128,$1100,$1184,$11C4,$118A,$118E,$1182,$11A8,$1182,$1188
.dw $1128,$1100,$118A,$11A6,$11A2,$1188,$1100,$1182,$11AC,$1188,$1100,$11AA,$1182,$11E0,$11A2,$11AA
.dw $1100,$118C,$11AE,$11C4,$1100,$11A2,$11AC,$118C,$11AE,$11C4,$11AA,$1182,$11C8,$11A2,$11AE,$11AC
.dw $1100,$1182,$1184,$11AE,$11CA,$11C8,$1100,$11C8,$11A0,$118A,$1100,$11C6,$11C0,$1186,$1100,$1182
.dw $11AC,$1188,$1100,$11C8,$11A0,$118A,$1100,$11C6,$11AC,$114E,$114C,$1148,$1160,$1162,$112C,$1100
.dw $1100,$1100,$11C8,$11AE,$1100,$11CC,$11A2,$11A8,$11A8,$118A,$1100,$118C,$11AE,$11C4,$1100,$11A0
.dw $11A2,$11C6,$1100,$1182,$11C6,$11C6,$118A,$11AA,$1184,$11A8,$118A,$11C4,$11C6,$1100,$11CE,$11A0
.dw $11A2,$1186,$11A0,$1128,$1100,$118A,$11CC,$118A,$11AC,$1100,$11CE,$11A2,$11C8,$11A0,$1100,$11C8
.dw $11A0,$118A,$11A2,$11C4,$1100,$118C,$11A8,$1182,$11CE,$11C6,$1128,$1100,$1186,$1182,$11AC,$1100
.dw $1186,$11AE,$11AA,$118A,$1100,$11A2,$11AC,$1100,$11C2,$11CA,$11A2,$11C8,$118A,$1100,$11A0,$1182
.dw $11AC,$1188,$11E2,$1100,$11C6,$11AE,$11AA,$118A,$11C8,$11A2,$11AA,$118A,$11C6,$112C,$1100,$1100
.dw $1100,$11C8,$11AE,$1100,$1184,$11E2,$11CA,$11CA,$1100,$118C,$11AE,$11C4,$1100,$1184,$11C6,$11AC
.dw $118A,$11C6,$1100,$11CE,$11A0,$11A2,$1186,$11A0,$1100,$11AA,$1182,$11A6,$118A,$11C6,$1100,$1188
.dw $118A,$1184,$11CA,$118E,$118E,$11A2,$11AC,$118E,$1100,$11C6,$11AC,$118A,$11C6,$1100,$1186,$11AE
.dw $1188,$118A,$1100,$11C6,$11AE,$1100,$11AA,$11CA,$1186,$11A0,$1100,$118A,$1182,$11C6,$11A2,$118A
.dw $11C4,$112C,$1100,$1100,$1100,$1100,$1182,$11AC,$1188,$1100,$11C8,$11AE,$1100,$1182,$11A8,$11A8
.dw $1100,$11C8,$11A0,$118A,$1100,$1186,$11AE,$11AA,$11C0,$11AE,$11C6,$118A,$11C4,$11C6,$1100,$11AE
.dw $118C,$1100,$11C8,$11A0,$118A,$1100,$11C6,$11AE,$11AC,$118E,$11C6,$1100,$11A2,$11AC,$1186,$11A8
.dw $11CA,$1188,$118A,$1188,$1166,$1100,$1188,$118A,$11C6,$11C0,$11A2,$11C8,$118A,$1100,$11C8,$11A0
.dw $118A,$1100,$11A8,$11A2,$11AA,$11A2,$11C8,$1182,$11C8,$11A2,$11AE,$11AC,$11C6,$1100,$11AE,$118C
.dw $1100,$11C8,$11A0,$118A,$1100,$11A0,$1182,$11C4,$1188,$11CE,$1182,$11C4,$118A,$1100,$11E2,$11AE
.dw $11CA,$1100,$11AA,$1182,$11AC,$1182,$118E,$118A,$1188,$1100,$11C8,$11AE,$1100,$1186,$11C4,$118A
.dw $1182,$11C8,$118A,$1100,$11A2,$11AC,$11C8,$118A,$11C4,$118A,$11C6,$11C8,$11A2,$11AC,$118E,$1100
.dw $11C0,$11A2,$118A,$1186,$118A,$11C6,$1100,$11AE,$118C,$1100,$11AA,$11CA,$11C6,$11A2,$1186,$112C
.dw $1100,$1100,$1100,$1100,$1100,$1100,$1100,$1100,$1100,$1100,$1100,$1100,$1100,$1100,$1100,$1100
.dw $1100,$1100,$1100,$1100,$1100,$1100,$1100,$1100,$112E,$11AA,$11A2,$1186,$1128,$1100,$1144,$1140
.dw $1142,$1140,$1128,$1144,$1140,$1142,$1144,$1100,$1100,$1100,$1100,$1100,$1100,$1100,$1100,$1100
.dw $1100,$1100,$1100,$1100,$1100,$1100,$1100,$1100,0
 .ends
 
 .bank 30
 .section "bank30data"
 song32lo: .incbin "vgp\song32.vgp" read $8000  
 .ends
 
 .bank 31
 .section "bank31data"
  song4: .incbin "vgp\song4.vgp"
 .ends
 .org $7ffd
 .db "mic"

 

