; VGM (SN76489) player for the SPC-700
; Extended version used in "The 700 Club" musicdisk
; Second edition
; /Mic, 2010, 2012
; 
; Assemble with wla-dx


.MEMORYMAP                     
  SLOTSIZE $10000             
  DEFAULTSLOT 0                
  SLOT 0 $0                
.ENDME         

.ROMBANKSIZE $10000              
.ROMBANKS 1    


.INCLUDE "dsp.inc"
.INCLUDE "filter_ids.inc"

.DEFINE TONE_GAIN 	80 
.DEFINE NOISE_GAIN 	44 
.DEFINE PER_NOISE_GAIN 	90 


; DP variables
.DEFINE VGM_PTR 	$10
.DEFINE TONE0_LATCH 	$12
.DEFINE TONE1_LATCH 	$14
.DEFINE TONE2_LATCH 	$16
.DEFINE NOISE_LATCH 	$18
.DEFINE VOL0_LATCH 	$1A
.DEFINE VOL1_LATCH 	$1C
.DEFINE VOL2_LATCH 	$1E
.DEFINE VOL3_LATCH 	$20
.DEFINE LATCHED_REG 	$22
.DEFINE TEMP 		$23
.DEFINE TEMP2 		$24
.DEFINE REG_BIT 	$25
.DEFINE FREQ_PTR 	$26
.DEFINE DELAY 		$28
.DEFINE BLOCK_SIZE	$30
.DEFINE FLAGS		$32
.DEFINE NUM_FLAGS	$33
.DEFINE PVAL		$34
.DEFINE SAMPLE		$36
.DEFINE MVOL		$38
.DEFINE LAST_BYTE	$39
.DEFINE PLAYING		$3A
.DEFINE OLDNGAIN	$3B
.DEFINE SENDLOOPS	$3C
.DEFINE CHN0TOGGLE	$40
.DEFINE CHN1TOGGLE	$41
.DEFINE CHN2TOGGLE	$42
.DEFINE CHN3TOGGLE	$43
.DEFINE ECHOFLAG	$47
.DEFINE FILTERNUM	$48
.DEFINE VERBATIM	$49
.DEFINE SAMPLE_DRIFT	$4A


; Commands sent by the S-CPU
.DEFINE CMD_PLAY		$21
.DEFINE CMD_PAUSE		$22
.DEFINE	CMD_STOP		$23
.DEFINE CMD_SEND_LOOP_MSGS	$24
.DEFINE CMD_RESTART_SONG	$25
.DEFINE	CMD_SET_MVOL 		$30
.DEFINE CMD_TOGGLE_CHN		$40
.DEFINE CMD_TOGGLE_ECHO		$50


.BANK 0
.ORGA 0
 nop

.ORGA ECHOFLAG
 .db $20
 .db FILTER_NONE
 
.ORGA $80
 filter_volume_table:
.DEFINE FILTER_VOLUME_TABLE $80
 .db 0,6,13,20,27,34,40,47,54,61,68,74,81,88,95,102

.ORGA $A0
 volume_table:
.DEFINE VOLUME_TABLE $A0
 .db 0,8,16,24,32,40,48,56,64,72,80,88,96,104,112,120

 
filters:
.DEFINE FILTERS $B0
 .db $7f,$00,$00,$00,$00,$00,$00,$00	; flat response
 .db $ff,$07,$15,$22,$22,$15,$07,$ff	; low pass
 .db $ff,$07,$15,$22,$22,$15,$07,$ff	; low pass
 .db $02,$fe,$ed,$22,$22,$ed,$fe,$02	; high pass
					; the 5th filter is "none", so no coefficients need
					; to be specified.

filter_settings:
.DEFINE FILTER_SETTINGS $D0
;    EDL, EFB, EVOL, FLAG
 .db 5,   48,   30,  $00
 .db 0,    0,   36,  $00		
 .db 5,   72,   31,  $00
 .db 0,    0,   94,  $00
 .db 0,    0,    0,  $20
 
 
.ORGA $0100

; Pause/unpause playback
pause:
	cmp		PLAYING,#0
	bne		+
	; We were in a paused state; key on all channels
	mov		SPC_DSP_ADDR,#DSP_KOF
	mov		SPC_DSP_DATA,#$00
	mov		PLAYING,#1
	nop
	nop
	nop	
	mov		SPC_DSP_ADDR,#DSP_KON
	mov		SPC_DSP_DATA,#$F
	ret
+:
	; We were in a playing state; key off all channels
	mov		SPC_DSP_ADDR,#DSP_KOF
	mov		SPC_DSP_DATA,#$FF
	mov		PLAYING,#0
	nop
	nop
	nop	
	mov		SPC_DSP_ADDR,#DSP_KON
	mov		SPC_DSP_DATA,#$0
	ret
	
	
prepare:
	mov		SPC_DSP_ADDR,#DSP_KOF
	mov		SPC_DSP_DATA,#$00

	mov		SPC_DSP_ADDR,#DSP_SRCN3
	mov		SPC_DSP_DATA,#1
	
	; No frequency modulation	
	mov		SPC_DSP_ADDR,#DSP_PMON
	mov		SPC_DSP_DATA,#0

	; Sample pointers are located at $0200
	mov		SPC_DSP_ADDR,#DSP_DIR
	mov		SPC_DSP_DATA,#2
		
	; Enable noise for channel 3	
	mov		SPC_DSP_ADDR,#DSP_NON
	mov		SPC_DSP_DATA,#8
	
	; Key on channels 0-3	
	mov		SPC_DSP_ADDR,#DSP_KON
	mov		SPC_DSP_DATA,#$F
	
	mov		TONE0_LATCH,#0
	mov		TONE1_LATCH,#0
	mov		TONE2_LATCH,#0
	mov		NOISE_LATCH,#0
	mov		VOL0_LATCH,#0
	mov		VOL1_LATCH,#0
	mov		VOL2_LATCH,#0
	mov		VOL3_LATCH,#0
	mov		LATCHED_REG,#0
	mov		NUM_FLAGS,#0
	
	mov		FREQ_PTR,#0

	; The VGM file starts with the standard 64-byte VGM header.
	mov		VGM_PTR,#<(vgm_file+64)
	mov		VGM_PTR+1,#>(vgm_file+64)
	
	mov		PLAYING,#1
	ret


; Store the current volumes in ports 2 and 3 (read by the S-CPU)
update_vol:
	mov		a,VOL1_LATCH 	; 3
	xcn		a		; 5
	mov		TEMP,a		; 4
	mov		a,VOL0_LATCH	; 3
	or		a,TEMP		; 3
	mov		SPC_PORT2,a	; 4
	
	mov		a,VOL3_LATCH	; 3
	xcn		a		; 5
	mov		TEMP,a		; 4
	mov		a,VOL2_LATCH	; 3
	or		a,TEMP		; 3
	mov		SPC_PORT3,a	; 4
	ret				; 5


; Restart from the song's loop point	
loop_song:
	cmp		SENDLOOPS,#0
	beq		+
	mov		a,LAST_BYTE
	clrc
	adc		a,#1
	mov		SPC_PORT0,a
+:
	mov		a,#<(vgm_file+28)
	mov		y,#>(vgm_file+28)
	movw		VGM_PTR,ya
	mov		a,!vgm_file+28
	mov		y,!vgm_file+29
	bne		+
	and		a,#$FF
	bne		+
	mov		y,#0
	mov		a,#$24
+:
	addw		ya,VGM_PTR
	movw		VGM_PTR,ya		; VGM_PTR = &vgm_file[0x1C] + vgm_file[0x1C];  // The word at 0x1C holds the loop offset
	mov		NUM_FLAGS,#0		; Reset the decompression flags
	jmp		!play	

compressed_long_wait:
	eor		TEMP,#$A0
	mov		TEMP+1,#>vgm_file
	mov		a,[TEMP+x]
	mov		DELAY,a
	mov		y,#1
	mov		a,[TEMP]+y
	mov		DELAY+1,a
	jmp		!long_wait_2
	
	
noise2_table:
.db $18,$18,$15,$13,$12,$11,$10,$10
.db $0F,$0F,$0E,$0E,$0D,$0D,$0D,$0C
.db $0C,$0C,$0C,$0B,$0B,$0B,$0B,$0A
.db $0A,$0A,$0A,$0A,$0A,$09,$09,$08

per_noise_table:
.DW 1638, 819, 410


.ORGA $0200

; Sample pointers
.DW sample0,sample0, sample1,sample1, sample2,sample2, sample3,sample3, sample4,sample4

sample0:
.DB $b3,$88,$88,$88,$88,$77,$77,$77,$77

sample1:
; 6.25% duty cycle square wave. Used for "perodic noise"
.DB $b3,$97,$99,$99,$99,$99,$99,$99,$99

; 6.25% (approx) duty cycle square wave at 1/4 frequency
sample2: 
.DB $b2,$77,$77,$99,$99,$99,$99,$99,$99,$b2,$99,$99,$99,$99,$99,$99,$99,$99,$b2,$99,$99,$99,$99,$99,$99,$99,$99,$b2,$99,$99,$99,$99,$99,$99,$99,$99

; 50% duty cycle square wave at 1/4 frequency
sample3:
.DB $b2,$88,$88,$88,$88,$88,$88,$88,$88,$b2,$88,$88,$88,$88,$88,$88,$88,$88,$b2,$77,$77,$77,$77,$77,$77,$77,$77,$b3,$77,$77,$77,$77,$77,$77,$77,$77

; 6.25% duty cycle square wave at 1/8 frequency
sample4: 
.DB $b2,$77,$77,$77,$77,$99,$99,$99,$99,$b2,$99,$99,$99,$99,$99,$99,$99,$99,$b2,$99,$99,$99,$99,$99,$99,$99,$99,$b2,$99,$99,$99,$99,$99,$99,$99,$99
.DB $b2,$99,$99,$99,$99,$99,$99,$99,$99,$b2,$99,$99,$99,$99,$99,$99,$99,$99,$b2,$99,$99,$99,$99,$99,$99,$99,$99,$b3,$99,$99,$99,$99,$99,$99,$99,$99


tone_vol:
.DB 120,99,79,63,51,41,33,27,22,18,15,13,11,9,6,0


.ORGA $0300

start:
	clrp
	mov		x,#$F0
	mov 		sp,x
	
	mov		SPC_DSP_ADDR,#DSP_EDL
	mov		SPC_DSP_DATA,#0
	mov		SPC_DSP_ADDR,#DSP_FLG
	mov		SPC_DSP_DATA,#$20		; Disable echo
	
	mov		MVOL,#112
	
	mov		SENDLOOPS,#0
	
	mov		CHN0TOGGLE,#$ff
	mov		CHN1TOGGLE,#$ff
	mov		CHN2TOGGLE,#$ff
	mov		CHN3TOGGLE,#$ff
	
	mov		LAST_BYTE,#$CD
	
	call		!stop

	mov		SPC_PORT0,LAST_BYTE

play:
	; Has a new command arrived from the S-CPU?
	mov		a,LAST_BYTE  		;3
	cmp		a,SPC_PORT0  		;3
	bne		+            		;2
	jmp		!no_new_command 	;3	  Nope
+:
	mov		a,SPC_PORT1
	mov		LAST_BYTE,SPC_PORT0
	cmp		a,#CMD_PLAY			; Was it a Play command?
	bne		+
	call		!prepare
	jmp		!processed_command
+:
	cmp		a,#CMD_PAUSE			; Was it a Pause command?
	bne		+
	call		!pause
	bra		processed_command
+:
	cmp		a,#CMD_TOGGLE_ECHO
	bne		+
	inc		FILTERNUM
	call		!update_filter
	bra		processed_command
+:	
	cmp		a,#CMD_STOP			; Was it a Stop command?
	bne		+	
	call		!stop
	jmp		!$FFC0
+:
	cmp		a,#CMD_SEND_LOOP_MSGS		; Was it a Send Loop Messages command?
	bne		+
	mov		a,SENDLOOPS
	eor		a,#1
	mov		SENDLOOPS,a
	bra		processed_command
+:
	mov		TEMP,a
	and		a,#$F0
	cmp		a,#CMD_SET_MVOL			; Was it a Set Master Volume command?
	bne		+
	mov		a,TEMP
	and		a,#$F
	or		a,ECHOFLAG
	mov		x,a
	mov		a,FILTER_VOLUME_TABLE+x
	mov		MVOL,a				; MVOL = (command & 0xF) * 8; // Gives a range of 0-120
	mov		SPC_DSP_ADDR,#DSP_MVOLL
	mov		SPC_DSP_DATA,MVOL
	mov		SPC_DSP_ADDR,#DSP_MVOLR
	mov		SPC_DSP_DATA,MVOL	
	bra		processed_command
+:	
	cmp		a,#CMD_TOGGLE_CHN		; Was it a Toggle Channel command?
	bne		+
	and		TEMP,#$F
	mov		a,TEMP
	xcn		a
	or		a,#DSP_GAIN0
	mov		TEMP2,a
	cmp		a,#DSP_GAIN3
	beq		toggle_noise
	mov		x,TEMP
	mov		a,CHN0TOGGLE+x
	eor		a,#255
	mov		CHN0TOGGLE+x,a
	and		a,#TONE_GAIN			; CHNxTOGGLE ^= 0xFF; DSP_GAINx = TONE_GAIN & CHNxTOGGLE;
	mov		SPC_DSP_ADDR,TEMP2
	mov		SPC_DSP_DATA,a
	;mov		SPC_PORT0,LAST_BYTE
	;bra		+
	bra		processed_command
toggle_noise:						; Channel 3 (noise) has its own gain, which is why it's handled separately
	eor		CHN3TOGGLE,#255
	mov		a,CHN3TOGGLE
	and		a,OLDNGAIN
	mov		SPC_DSP_ADDR,#DSP_GAIN3
	mov		SPC_DSP_DATA,a
	mov		SPC_PORT0,LAST_BYTE
processed_command:
	mov		SPC_PORT0,LAST_BYTE	
+:
; There are no new commands from the S-CPU. Are we in Playing or Paused/Stopped state?
no_new_command:
	; =11 cycles spent=
	cmp		PLAYING,#0 ;5
	bne		+  ;2/4
	jmp		!play  
+:
        ; =20 cycles spent=
	mov		x,#0  ;2

	; Perform decompression..
	
	mov		a,NUM_FLAGS		; 3. any flags left?
	bne		+			; 2/4
	mov		NUM_FLAGS,#7		; 5
	mov		a,[VGM_PTR+x]		; 6. load a new flags byte
	mov		FLAGS,a			; 3
	mov		VERBATIM,a		; 3
	incw		VGM_PTR			; 6
+:
	; =32 cycles spent (average)=
	dec		NUM_FLAGS		; 4
	ror		FLAGS			; 4. put the next flag in C
	bcc		+			; 2/4
	call		!psg_param		; 8. the flag was set; this is a PSG write command
	jmp		!play
	
+:
	; =44 cycles spent=
	bbc		VERBATIM .7,++		; 5/7
	mov		a,[VGM_PTR+x]		; 6. the was flag clear; this is not a PSG write command
	incw		VGM_PTR			; 6

	clrc					; 2
	adc		SAMPLE_DRIFT,#3		; 5
	
	mov		TEMP,a			; 4. save the command byte
	and		a,#$F0			; 2
	cmp		a,#$70			; 2. first check if it's a short wait command since we want the lowest latency in processing them
	beq		short_wait		; 2/4
	cmp		a,#$90
	beq		+++	
	mov		a,TEMP	
	cmp		a,#$4E			; $4E is used a a NOP command to pad compression runs when needed
	bne		+
	jmp		!play
+:
	cmp		a,#$66			; loop
	bne		+
	jmp		!loop_song
+:
	cmp		a,#$4F			; set gamegear stereo parameter
	beq		gg_stereo_param
--:
	cmp		a,#$62			; wait one ntsc frame (1/60 s)
	beq		wait_frame_ntsc
	cmp		a,#$63			; wait one pal frame (1/50 s)
	beq		wait_frame_pal
	cmp		a,#$61			; wait xxyy samples
	beq		long_wait
	cmp		a,#$67			; data block
	bne		+
	mov		x,#3
	mov		a,[VGM_PTR+x]
	mov		y,a
	dec		x
	mov		a,[VGM_PTR+x]
	clrc
	adc		a,#4
	addw		ya,VGM_PTR
	movw		VGM_PTR,ya
+:

	; All other commands are unhandled and assumed to be 3 bytes long
	incw		VGM_PTR
	incw		VGM_PTR
	jmp		!play
++:
	mov		a,!vgm_file+36		; 4
	clrc					; 2
	adc		SAMPLE_DRIFT,#3		; 5
	bra		--			; 4
+++:
	jmp		!compressed_long_wait
	

; Wait n/44100 s  (n = [1..16])
short_wait:
	and		TEMP,#$F
	mov		x,TEMP
	beq		+
short_wait_2:
	; 23 cycles/iteration
	inc		!vgm_file+20	; 5
	dec		!vgm_file+20	; 5
	cmp		x,VERBATIM	; 3
	and		a,#$0F		; 2
	and		a,#$0F		; 2
	dec		a		; 2
	bne		short_wait_2	; 4
+:
	jmp		!play


; TODO: Handle GG stereo settings
gg_stereo_param:
	incw		VGM_PTR
	jmp		!play


; Wait 1/60 s
wait_frame_ntsc:
	mov		DELAY,#$DF
	mov		DELAY+1,#2
	bra		long_wait_2


; Wait 1/50 s
wait_frame_pal:
	mov		DELAY,#$72
	mov		DELAY+1,#3
	bra		long_wait_2


long_wait:
	mov		a,[VGM_PTR+x]	; 6
	mov		DELAY,a		; 4
	incw		VGM_PTR		; 6
	mov		a,[VGM_PTR+x]	; 6
	mov		DELAY+1,a	; 4
	incw		VGM_PTR		; 6
long_wait_2:
	call		!update_vol	; 8 + 49
	clrc		
	adc		SAMPLE_DRIFT,#4
	movw		ya,DELAY
	subw		ya,SAMPLE_DRIFT
	bmi		+
	beq		+
	movw		DELAY,ya
	; 23 cycles/iteration
-:
	inc		!vgm_file+20	; 5
	mov		a,!vgm_file+20	; 4
	mov		a,!vgm_file+20	; 4
	decw		DELAY		; 6
	bne		-		; 4
+:
	mov		SAMPLE_DRIFT,#0
	jmp		!play
	
	

; A value is being written to the PSG	
psg_param:
	; 50 cycles spent
	mov		a,[VGM_PTR+x]		; 6. read the parameter byte
	incw		VGM_PTR			; 6
	mov		TEMP,a			; 3
	bbs		TEMP .7,latch_data	; 5/7
	mov		a,LATCHED_REG		; 3
	cmp		a,#6			; 2
	bcs		+			; 2/4
	; 77 cycles spent
	; Set high 6 bits of tone register
	mov		x,a			; 2
	mov		a,TEMP			; 3
	xcn		a			; 5
	and		a,#$F0			; 2
	mov		TEMP2,a			; 3
	mov		a,TONE0_LATCH+x		; 4
	and		a,#$F			; 2. save bits 0-3
	or		a,TEMP2			; 3. replace bits 4-7 of the tone period with bits 0-3 of the parameter byte
	mov		TONE0_LATCH+x,a		; 5
	inc		x			; 2
	mov		a,TEMP			; 3
	xcn		a			; 5
	and		a,#3			; 2
	mov		TONE0_LATCH+x,a		; 4. set bits 8-9 of the tone period to bits 4-5 of the parameter byte
	jmp		!tone_reg_updated	; 3. the tone register has been updated, so we should update the channels frequency on the S-DSP
	; =125 cycles spent
+:
	; =79 cycles spent
	mov		x,a			; 2
	mov		a,TEMP			; 3
	and		a,#$F			; 2
	mov		TONE0_LATCH+x,a		; 4
check_vol_noise:
	bbs		LATCHED_REG .3,volume_reg_updated  ; 5/7
	jmp		!noise_reg_updated	; 3
	; =98 cycles spent
latch_data:
	; =72 cycles spent
	mov 		a,TEMP			; 3
	xcn		a			; 5
	and		a,#$7			; 2
	mov		TEMP2,a			; 3
	lsr		a			; 2
	and		TEMP2,#1		; 5
	asl		TEMP2			; 4
	asl		TEMP2			; 4
	or		a,TEMP2			; 3
	asl		a			; 2
	mov		LATCHED_REG,a		; 3
	mov		x,a			; 2
	and		TEMP,#$F		; 5
	mov		a,TONE0_LATCH+x		; 4
	and		a,#$F0			; 2
	or		a,TEMP			; 3
	mov		TONE0_LATCH+x,a		; 5
	cmp		LATCHED_REG,#6		; 5
	bcs		+ 			; 2/4
	inc		SAMPLE_DRIFT		; 4
	bra		tone_reg_updated	; 4
	; =144 cycles spent=
+:
	; =138 cycles spent=
	inc		SAMPLE_DRIFT
	inc		SAMPLE_DRIFT
	bra		check_vol_noise
	
volume_reg_updated:
	; 97 cycles spent
	mov		x,LATCHED_REG		; 3
	mov		a,TONE0_LATCH+x		; 4
	mov 		x,a			; 2
	mov 		a,!tone_vol+x		; 5
	mov		TEMP,a			; 4
	mov		a,LATCHED_REG		; 3
	lsr		a			; 2
	and		a,#3			; 2
	xcn		a			; 5		
	; A will now contain $00, $10, $20 or $30 - i.e. DSP_VOL0L, DSP_VOL1L, DSP_VOL2L or DSP_VOL3L
	mov		SPC_DSP_ADDR,a		; 4
	mov		SPC_DSP_DATA,TEMP	; 5. set left volume
	inc		SPC_DSP_ADDR		; 4. move to the next register (VOLxR)
	mov		SPC_DSP_DATA,TEMP	; 5. set right volume
	clrc					; 2
	adc		SAMPLE_DRIFT,#7		; 5
	ret					; 5
	; 157 cycles spent
	

; TODO: Handle constant output (psgPeriod <= 1)
tone_reg_updated:
	; =125 cycles spent=
	mov		SAMPLE,#0		; 5
	mov		a,LATCHED_REG		; 3
	lsr		a			; 2
	and		a,#3			; 2
	clrc					; 2
	xcn		a			; 5
	adc		a,#DSP_P0L		; 2
	mov		TEMP,a			; 4. TEMP = DSP_P0L, DSP_P1L or DSP_P2L
	mov		x,LATCHED_REG		; 3
	mov		a,TONE0_LATCH+x		; 4
	asl		a			; 2
	mov		y,a			; 2. Y = psgPeriodLo << 1
	mov		a,TONE0_LATCH+1+x	; 4
	rol		a			; 2. A = (psgPeriodHi << 1) | ((psgPeriodLo & 0x80) >> 7)
	and		a,#7			; 2. the frequency table is $800 bytes long
	clrc					; 2
	adc		a,#8 ;7			; 2. ..and it starts at $700 
	mov		FREQ_PTR+1,a		; 4
	mov		a,[FREQ_PTR]+y		; 6. load low byte of S-DSP period
	mov		PVAL,a			; 4
	inc		y			; 2	
	mov		a,[FREQ_PTR]+y		; 6. load high byte of S-DSP period
	mov		PVAL+1,a		; 4
	; Bit 15 of PVAL is used as a flag. If it's set then a sample 4x as wide should be used.
	; P has already been quadrupled in the table for these values.
	; =199 cycles spent=
	bpl		+			; 2/4
	and		PVAL+1,#$3F		; 5
	mov		SAMPLE,#3		; 5
+:
	mov		SPC_DSP_ADDR,TEMP	; 5
	mov		SPC_DSP_DATA,PVAL	; 5. write to DSP_PxL
	inc		SPC_DSP_ADDR		; 4
	mov		SPC_DSP_DATA,PVAL+1	; 5. write to DSP_PxH
	inc		SPC_DSP_ADDR		; 4
	mov		SPC_DSP_DATA,SAMPLE	; 5. write to DSP_SRCNx

	; If tone2 was updated and the noise channel is set to "periodic noise" with tone2 as the driving signal, then we
	; need to update channel 3 as well.
	; =235 cycles spent=	
	cmp		LATCHED_REG,#4		; 5
	bne		+			; 2/4
	cmp		NOISE_LATCH,#3		; 5
	bne		+			; 2/4
	clrc					; 2
	adc		SAMPLE_DRIFT,#6		; 5
	jmp		!periodic_noise		; 3
+:
	clrc					; 2
	adc		SAMPLE_DRIFT,#11	; 5
	ret					; 5
	
	
noise_reg_updated:
	; 98 cycles spent
	bbc		NOISE_LATCH .2,periodic_noise  ;5/7
	mov		SPC_DSP_ADDR,#DSP_NON		; 5
	mov		SPC_DSP_DATA,#8			; 5
	mov		SPC_DSP_ADDR,#DSP_GAIN3		; 5
	mov		a,#NOISE_GAIN			; 2
	and		a,CHN3TOGGLE			; 3
	mov		SPC_DSP_DATA,a			; 4
	mov		OLDNGAIN,#NOISE_GAIN		; 5
	mov		a,NOISE_LATCH			; 3
	and		a,#3				; 2
	cmp		a,#3				; 2
	beq		+				; 2/4
	mov		TEMP,a				; 4
	asl		a				; 2
	clrc						; 2
	adc		a,TEMP				; 3
	mov		TEMP,a				; 4
	; 156 cycles spent
	; We want to set the noise clock to $1B - (param*3), i.e. 6400 Hz / (param+1). $20 is added to keep echo disabled
	;mov		a,#$3B				; 2			
	mov		a,ECHOFLAG
	or		a,#$1B
	setc						; 2
	sbc		a,TEMP				; 3
	mov		SPC_DSP_ADDR,#DSP_FLG		; 5
	mov		SPC_DSP_DATA,a			; 4
	clrc						; 2
	adc		SAMPLE_DRIFT,#8			; 5
	ret						; 5
	
+:
	; 143 cycles spent
	mov		TEMP,TONE2_LATCH		; 5
	mov		a,TONE2_LATCH+1			; 3
	rol		TEMP				; 4
	rol		a				; 2
	rol		TEMP				; 4
	rol		a				; 2	
	rol		TEMP				; 4
	rol		a				; 2
	and		a,#$1F				; 2
	mov		x,a				; 2
	mov		a,!noise2_table+x		; 5
	or		a,ECHOFLAG			; 3
	mov		SPC_DSP_ADDR,#DSP_FLG		; 5
	mov		SPC_DSP_DATA,a			; 4
	clrc						; 2
	adc		SAMPLE_DRIFT,#9			; 5
	ret						; 5
periodic_noise:
	; 105 cycles spent
	mov		SPC_DSP_ADDR,#DSP_NON	; 5
	mov		SPC_DSP_DATA,#0		; 5
	mov		SPC_DSP_ADDR,#DSP_GAIN3	; 5
	mov		a,#PER_NOISE_GAIN	; 2
	and		a,CHN3TOGGLE		; 3
	mov		SPC_DSP_DATA,a		; 4
	mov		OLDNGAIN,#TONE_GAIN	; 5
	mov		a,NOISE_LATCH		; 2
	and		a,#3			; 2
	cmp		a,#3			; 2
	beq		tone2_freq_pernoise	; 2/4
	asl		a			; 2
	mov		x,a			; 2
	mov		a,!per_noise_table+x	; 5
	mov		SPC_DSP_ADDR,#DSP_P3L	; 5
	mov		SPC_DSP_DATA,a		; 4
	inc		SPC_DSP_ADDR		; 4
	mov		a,!per_noise_table+1+x	; 5
	mov		SPC_DSP_DATA,a		; 4
	clrc					; 2
	adc		SAMPLE_DRIFT,#8
	ret					; 5
	
; "Periodic noise" with tone2's period as the counter reload value
tone2_freq_pernoise:
	; 144 cycles spent
	mov		a,TONE2_LATCH		; 3
	asl		a			; 2
	mov		y,a			; 2. Y = psgPeriodLo << 1
	mov		a,TONE2_LATCH+1		; 3
	rol		a			; 2. A = (psgPeriodHi << 1) | ((psgPeriodLo & 0x80) >> 7)
	and		a,#7			; 2. the frequency table is $800 bytes long
	clrc					; 2
	adc		a,#$10 ;F			; 2. ..and it starts at $F00 
	mov		FREQ_PTR+1,a		; 4

	mov		SAMPLE,#1		; 5	
	mov		a,[FREQ_PTR]+y		; 6. load low byte of S-DSP period
	mov		PVAL,a			; 4
	inc		y			; 2
	mov		a,[FREQ_PTR]+y		; 6. load high byte of S-DSP period
	mov		PVAL+1,a		; 4	
	; Bit 15 and 14 of PVAL are used as flags. If bit 15 set then a sample 4x as wide should be used.
	; If both bit 14 and 15 are set then a sample 8x as wide should be used.
	; P has already been prescaled in the table for these values.
	; 193 cycles spent
	bpl		+			; 2/4
	mov		SAMPLE,#2		; 5
	bbc		PVAL+1 .6,+		; 5/7
	mov		SAMPLE,#4		; 5
+:
	and		PVAL+1,#$7F		; 5
	mov		SPC_DSP_ADDR,#DSP_P3L	; 5
	mov		SPC_DSP_DATA,PVAL	; 5	
	inc		SPC_DSP_ADDR		; 4
	mov		SPC_DSP_DATA,PVAL+1	; 5		
	inc		SPC_DSP_ADDR		; 4
	mov		SPC_DSP_DATA,SAMPLE	; 5
	clrc					; 2
	adc		SAMPLE_DRIFT,#11	; 5
	ret					; 5


	


stop:
	mov		PLAYING,#0

	mov		SAMPLE_DRIFT,#0
	mov		SAMPLE_DRIFT+1,#0

	mov		SPC_DSP_ADDR,#DSP_FLG
	mov		SPC_DSP_DATA,#$20
	
	; Clear per-channel DSP registers (VOL, P, SRCN, ADSR, GAIN)
	mov		x,#0
	mov		y,#$80
clear_dsp:
	mov		a,x
	and		a,#$77
	mov		SPC_DSP_ADDR,a
	mov		SPC_DSP_DATA,#0	
	inc		x
	dbnz		y,clear_dsp

	; Key off all channels
	mov		SPC_DSP_ADDR,#DSP_KOF
	mov		SPC_DSP_DATA,#$FF

	call		!update_filter
	
	; Set master volume	
	mov		SPC_DSP_ADDR,#DSP_MVOLL
	mov		SPC_DSP_DATA,MVOL
	mov		SPC_DSP_ADDR,#DSP_MVOLR
	mov		SPC_DSP_DATA,MVOL

	; Set gain levels. The gain for the noise channel needs to be much lower than for
	; the tone channels.
	mov		SPC_DSP_ADDR,#DSP_GAIN0
	mov		SPC_DSP_DATA,#TONE_GAIN
	mov		SPC_DSP_ADDR,#DSP_GAIN1
	mov		SPC_DSP_DATA,#TONE_GAIN
	mov		SPC_DSP_ADDR,#DSP_GAIN2
	mov		SPC_DSP_DATA,#TONE_GAIN
	mov		SPC_DSP_ADDR,#DSP_GAIN3
	mov		SPC_DSP_DATA,#NOISE_GAIN
	mov		OLDNGAIN,#NOISE_GAIN

	mov		VOL0_LATCH,#0
	mov		VOL1_LATCH,#0
	mov		VOL2_LATCH,#0
	mov		VOL3_LATCH,#0
	
	call		!update_vol
	
	mov		SPC_CTRL,#$B0
	ret




update_filter:
	cmp		FILTERNUM,#FILTER_INVALID
	bne		+
	mov		FILTERNUM,#FILTER_FLAT_DELAY
+:

	mov		SPC_DSP_ADDR,#DSP_FLG
	mov		SPC_DSP_DATA,#$20
	
	mov		a,FILTERNUM
	asl		a
	asl		a
	mov		x,a
	
	mov		SPC_DSP_ADDR,#DSP_ESA
	mov		SPC_DSP_DATA,#$CF

	mov		a,FILTER_SETTINGS+1+x
	mov		SPC_DSP_ADDR,#DSP_EFB
	mov		SPC_DSP_DATA,a
	
	mov		a,FILTER_SETTINGS+2+x
	mov		SPC_DSP_ADDR,#DSP_EVOLL
	mov		SPC_DSP_DATA,a
	mov		SPC_DSP_ADDR,#DSP_EVOLR
	mov		SPC_DSP_DATA,a

	mov		a,FILTER_SETTINGS+x	
	mov		SPC_DSP_ADDR,#DSP_EDL
	mov		SPC_DSP_DATA,a

	mov		a,FILTER_SETTINGS+3+x
	mov		SPC_DSP_ADDR,#DSP_FLG
	mov		SPC_DSP_DATA,a
	mov		ECHOFLAG,a
	
	mov		y,#$0F
	cmp		a,#$20
	bne		+
	mov		y,#0
+:
	mov		SPC_DSP_ADDR,#DSP_EON
	mov		SPC_DSP_DATA,y

	mov		a,x
	asl		a
	mov		x,a

	mov		a,FILTERS+0+x
	mov		SPC_DSP_ADDR,#DSP_COEF0
	mov		SPC_DSP_DATA,a
	mov		a,FILTERS+1+x
	mov		SPC_DSP_ADDR,#DSP_COEF1
	mov		SPC_DSP_DATA,a
	mov		a,FILTERS+2+x
	mov		SPC_DSP_ADDR,#DSP_COEF2
	mov		SPC_DSP_DATA,a
	mov		a,FILTERS+3+x
	mov		SPC_DSP_ADDR,#DSP_COEF3
	mov		SPC_DSP_DATA,a
	mov		a,FILTERS+4+x
	mov		SPC_DSP_ADDR,#DSP_COEF4
	mov		SPC_DSP_DATA,a
	mov		a,FILTERS+5+x
	mov		SPC_DSP_ADDR,#DSP_COEF5
	mov		SPC_DSP_DATA,a
	mov		a,FILTERS+6+x
	mov		SPC_DSP_ADDR,#DSP_COEF6
	mov		SPC_DSP_DATA,a
	mov		a,FILTERS+7+x
	mov		SPC_DSP_ADDR,#DSP_COEF7
	mov		SPC_DSP_DATA,a
	ret	
	


.ORGA $0800
freq_table:
.INCLUDE "freqtb.inc"

freq_table2:
.INCLUDE "freqtb2.inc"

vgm_file:





